@@ -24,20 +24,19 @@
 
 #include "mainwindow.h"
 #include "ark_debug.h"
-#include "kerfuffle/archive_kerfuffle.h"
-#include "kerfuffle/createdialog.h"
-#include "kerfuffle/settingspage.h"
-#include "mimetypes.h"
+#include "archive_kerfuffle.h"
+#include "createdialog.h"
+#include "settingspage.h"
+#include "pluginmanager.h"
 #include "part/interface.h"
 
+#include <KParts/ReadWritePart>
 #include <KPluginFactory>
 #include <KMessageBox>
 #include <KLocalizedString>
 #include <KActionCollection>
 #include <KStandardAction>
 #include <KRecentFilesAction>
-#include <KEditToolBar>
-#include <KShortcutsDialog>
 #include <KService>
 #include <KSharedConfig>
 #include <KConfigDialog>
@@ -46,7 +45,6 @@
 #include <KXMLGUIFactory>
 
 #include <QApplication>
-#include <QDebug>
 #include <QDragEnterEvent>
 #include <QDragMoveEvent>
 #include <QFileDialog>
@@ -63,14 +61,7 @@ static bool isValidArchiveDrag(const QMimeData *data)
 MainWindow::MainWindow(QWidget *)
         : KParts::MainWindow()
 {
-    setXMLFile(QStringLiteral("arkui.rc"));
-
     setupActions();
-
-    resize(640, 480);
-
-    setAutoSaveSettings(QStringLiteral("MainWindow"));
-
     setAcceptDrops(true);
 }
 
@@ -136,14 +127,19 @@ void MainWindow::dragMoveEvent(QDragMoveEvent * event)
 
 bool MainWindow::loadPart()
 {
-    KPluginFactory *factory = 0;
-    KService::Ptr service = KService::serviceByDesktopName(QStringLiteral("ark_part"));
+    KPluginFactory *factory = Q_NULLPTR;
 
-    if (service) {
-        factory = KPluginLoader(service->library()).factory();
+    const auto plugins = KPluginLoader::findPlugins(QString(), [](const KPluginMetaData& metaData) {
+        return metaData.pluginId() == QStringLiteral("arkpart") &&
+               metaData.serviceTypes().contains(QStringLiteral("KParts/ReadOnlyPart")) &&
+               metaData.serviceTypes().contains(QStringLiteral("Browser/View"));
+    });
+
+    if (plugins.size() == 1) {
+        factory = KPluginLoader(plugins.first().fileName()).factory();
     }
 
-    m_part = factory ? static_cast<KParts::ReadWritePart*>(factory->create<KParts::ReadWritePart>(this)) : 0;
+    m_part = factory ? static_cast<KParts::ReadWritePart*>(factory->create<KParts::ReadWritePart>(this)) : Q_NULLPTR;
 
     if (!m_part) {
         KMessageBox::error(this, i18n("Unable to find Ark's KPart component, please check your installation."));
@@ -153,7 +149,10 @@ bool MainWindow::loadPart()
 
     m_part->setObjectName(QStringLiteral("ArkPart"));
     setCentralWidget(m_part->widget());
+
+    setupGUI(ToolBar | Keys | Save, QStringLiteral("arkui.rc"));
     createGUI(m_part);
+
     statusBar()->hide();
 
     connect(m_part, SIGNAL(busy()), this, SLOT(updateActions()));
@@ -177,11 +176,7 @@ void MainWindow::setupActions()
     m_recentFilesAction->setIconText(i18nc("action, to open an archive", "Open"));
     m_recentFilesAction->setToolTip(i18n("Open an archive"));
     m_recentFilesAction->loadEntries(KSharedConfig::openConfig()->group("Recent Files"));
-    connect(m_recentFilesAction, SIGNAL(triggered()),
-            this, SLOT(openArchive()));
 
-    KStandardAction::configureToolbars(this, SLOT(editToolbars()), actionCollection());
-    KStandardAction::keyBindings(this, SLOT(editKeyBindings()), actionCollection());
     KStandardAction::preferences(this, SLOT(showSettings()), actionCollection());
 }
 
@@ -193,39 +188,16 @@ void MainWindow::updateActions()
     m_recentFilesAction->setEnabled(!iface->isBusy());
 }
 
-void MainWindow::editKeyBindings()
-{
-    KShortcutsDialog dlg(KShortcutsEditor::AllActions, KShortcutsEditor::LetterShortcutsAllowed, this);
-    dlg.addCollection(actionCollection());
-    dlg.addCollection(m_part->actionCollection());
-
-    dlg.configure();
-}
-
-void MainWindow::editToolbars()
-{
-    KConfigGroup cfg(KSharedConfig::openConfig(), "MainWindow");
-    saveMainWindowSettings(cfg);
-
-    QPointer<KEditToolBar> dlg = new KEditToolBar(factory(), this);
-    dlg.data()->exec();
-
-    createGUI(m_part);
-
-    applyMainWindowSettings(KSharedConfig::openConfig()->group(QStringLiteral("MainWindow")));
-
-    delete dlg.data();
-}
-
 void MainWindow::openArchive()
 {
     Interface *iface = qobject_cast<Interface*>(m_part);
     Q_ASSERT(iface);
     Q_UNUSED(iface);
 
+    Kerfuffle::PluginManager pluginManager;
     auto dlg = new QFileDialog(this, i18nc("to open an archive", "Open Archive"));
 
-    dlg->setMimeTypeFilters(Kerfuffle::supportedMimeTypes());
+    dlg->setMimeTypeFilters(pluginManager.supportedMimeTypes());
     dlg->setFileMode(QFileDialog::ExistingFile);
     dlg->setAcceptMode(QFileDialog::AcceptOpen);
 
@@ -312,20 +284,26 @@ void MainWindow::newArchive()
 
     QPointer<Kerfuffle::CreateDialog> dialog = new Kerfuffle::CreateDialog(
         Q_NULLPTR, // parent
-        i18n("Create a new Archive"), // caption
+        i18n("Create New Archive"), // caption
         QUrl()); // startDir
 
-    dialog.data()->show();
-    dialog.data()->restoreWindowSize();
-
     if (dialog.data()->exec()) {
-        const QUrl saveFileUrl = dialog.data()->selectedUrls().first();
+        const QUrl saveFileUrl = dialog.data()->selectedUrl();
         const QString password = dialog.data()->password();
+        const QString fixedMimeType = dialog.data()->currentMimeType().name();
 
         qCDebug(ARK) << "CreateDialog returned URL:" << saveFileUrl.toString();
-        qCDebug(ARK) << "CreateDialog returned mime:" << dialog.data()->currentFilterMimeType().name();
+        qCDebug(ARK) << "CreateDialog returned mime:" << fixedMimeType;
 
         m_openArgs.metaData()[QStringLiteral("createNewArchive")] = QStringLiteral("true");
+        m_openArgs.metaData()[QStringLiteral("fixedMimeType")] = fixedMimeType;
+        if (dialog.data()->compressionLevel() > -1) {
+            m_openArgs.metaData()[QStringLiteral("compressionLevel")] = QString::number(dialog.data()->compressionLevel());
+        }
+        if (dialog.data()->volumeSize() > 0) {
+            qCDebug(ARK) << "Setting volume size:" << QString::number(dialog.data()->volumeSize());
+            m_openArgs.metaData()[QStringLiteral("volumeSize")] = QString::number(dialog.data()->volumeSize());
+        }
         m_openArgs.metaData()[QStringLiteral("encryptionPassword")] = password;
 
         if (dialog.data()->isHeaderEncryptionEnabled()) {
@@ -336,6 +314,8 @@ void MainWindow::newArchive()
 
         m_openArgs.metaData().remove(QStringLiteral("showExtractDialog"));
         m_openArgs.metaData().remove(QStringLiteral("createNewArchive"));
+        m_openArgs.metaData().remove(QStringLiteral("fixedMimeType"));
+        m_openArgs.metaData().remove(QStringLiteral("compressionLevel"));
         m_openArgs.metaData().remove(QStringLiteral("encryptionPassword"));
         m_openArgs.metaData().remove(QStringLiteral("encryptHeader"));
     }
