@@ -58,9 +58,8 @@ void ArchiveTest::testProperties_data()
     QTest::addColumn<QString>("expectedSubfolderName");
 
     // Test non-existent tar archive.
-    QString archivePath = QStringLiteral("/tmp/foo.tar.gz");
     QTest::newRow("non-existent tar archive")
-            << archivePath
+            << QStringLiteral("/tmp/foo.tar.gz")
             << QStringLiteral("foo")
             << false << false << false << Archive::Unencrypted
             << QString();
@@ -73,88 +72,76 @@ void ArchiveTest::testProperties_data()
             << QString();
 
     // Test dummy source code tarball.
-    archivePath = QFINDTESTDATA("data/code-x.y.z.tar.gz");
     QTest::newRow("dummy source code tarball")
-            << archivePath
+            << QFINDTESTDATA("data/code-x.y.z.tar.gz")
             << QStringLiteral("code-x.y.z")
             << false << false << true << Archive::Unencrypted
             << QStringLiteral("awesome_project");
 
-    archivePath = QFINDTESTDATA("data/simplearchive.tar.gz");
     QTest::newRow("simple compressed tar archive")
-            << archivePath
+            << QFINDTESTDATA("data/simplearchive.tar.gz")
             << QStringLiteral("simplearchive")
             << false << false << false << Archive::Unencrypted
             << QStringLiteral("simplearchive");
 
-    archivePath = QFINDTESTDATA("data/archivetest_encrypted.zip");
     QTest::newRow("encrypted zip, single entry")
-            << archivePath
+            << QFINDTESTDATA("data/archivetest_encrypted.zip")
             << QStringLiteral("archivetest_encrypted")
             << false << true << false << Archive::Encrypted
             << QStringLiteral("archivetest_encrypted");
 
-    archivePath = QFINDTESTDATA("data/archivetest_unencrypted.zip");
     QTest::newRow("simple zip, one unencrypted entry")
-            << archivePath
+            << QFINDTESTDATA("data/archivetest_unencrypted.zip")
             << QStringLiteral("archivetest_unencrypted")
             << false << true << false << Archive::Unencrypted
             << QStringLiteral("archivetest_unencrypted");
 
-    archivePath = QFINDTESTDATA("data/wget.rpm");
     QTest::newRow("rpm archive, no single folder")
-            << archivePath
+            << QFINDTESTDATA("data/wget.rpm")
             << QStringLiteral("wget")
             << true << false << false << Archive::Unencrypted
             << QStringLiteral("wget");
 
-    archivePath = QFINDTESTDATA("data/simplearchive.tar.bz2");
     QTest::newRow("bzip2-compressed tarball")
-            << archivePath
+            << QFINDTESTDATA("data/simplearchive.tar.bz2")
             << QStringLiteral("simplearchive")
             << false << false << false << Archive::Unencrypted
             << QStringLiteral("simplearchive");
 
-    archivePath = QFINDTESTDATA("data/simplearchive.tar.xz");
     QTest::newRow("xz-compressed tarball")
-            << archivePath
+            << QFINDTESTDATA("data/simplearchive.tar.xz")
             << QStringLiteral("simplearchive")
             << false << false << false << Archive::Unencrypted
             << QStringLiteral("simplearchive");
 
-    archivePath = QFINDTESTDATA("data/simplearchive.tar.lzma");
     QTest::newRow("lzma-compressed tarball")
-            << archivePath
+            << QFINDTESTDATA("data/simplearchive.tar.lzma")
             << QStringLiteral("simplearchive")
             << false << false << false << Archive::Unencrypted
             << QStringLiteral("simplearchive");
 
-    archivePath = QFINDTESTDATA("data/simplearchive.tar.Z");
     QTest::newRow("compress (.Z) tarball")
-            << archivePath
+            << QFINDTESTDATA("data/simplearchive.tar.Z")
             << QStringLiteral("simplearchive")
             << false << false << false << Archive::Unencrypted
             << QStringLiteral("simplearchive");
 
-    archivePath = QFINDTESTDATA("data/simplearchive.tar.lz");
     QTest::newRow("lzipped tarball")
-            << archivePath
+            << QFINDTESTDATA("data/simplearchive.tar.lz")
             << QStringLiteral("simplearchive")
             << false << false << false << Archive::Unencrypted
             << QStringLiteral("simplearchive");
 
-    archivePath = QFINDTESTDATA("data/simplearchive.tar.lzo");
     QTest::newRow("lzop-compressed tarball")
-            << archivePath
+            << QFINDTESTDATA("data/simplearchive.tar.lzo")
             << QStringLiteral("simplearchive")
             << false << false << false << Archive::Unencrypted
             << QStringLiteral("simplearchive");
 
     // Only run test for lrzipped tar if lrzip executable is found in path.
     if (!QStandardPaths::findExecutable(QStringLiteral("lrzip")).isEmpty()) {
-        archivePath = QFINDTESTDATA("data/simplearchive.tar.lrz");
         QTest::newRow("lrzipped tarball")
-                << archivePath
+                << QFINDTESTDATA("data/simplearchive.tar.lrz")
                 << QStringLiteral("simplearchive")
                 << false << false << false << Archive::Unencrypted
                 << QStringLiteral("simplearchive");
@@ -162,11 +149,34 @@ void ArchiveTest::testProperties_data()
         qDebug() << "lrzip executable not found in path. Skipping lrzip test.";
     }
 
+    // Only run test for lz4-compressed tar if lz4 executable is found in path.
+    if (!QStandardPaths::findExecutable(QStringLiteral("lz4")).isEmpty()) {
+        QTest::newRow("lz4-compressed tarball")
+                << QFINDTESTDATA("data/simplearchive.tar.lz4")
+                << QStringLiteral("simplearchive")
+                << false << false << false << Archive::Unencrypted
+                << QStringLiteral("simplearchive");
+    } else {
+        qDebug() << "lz4 executable not found in path. Skipping lz4 test.";
+    }
+
+    QTest::newRow("xar archive")
+            << QFINDTESTDATA("data/simplearchive.xar")
+            << QStringLiteral("simplearchive")
+            << true << false << false << Archive::Unencrypted
+            << QStringLiteral("simplearchive");
+
     QTest::newRow("mimetype child of application/zip")
             << QFINDTESTDATA("data/test.odt")
             << QStringLiteral("test")
             << false << true << false << Archive::Unencrypted
             << QStringLiteral("test");
+
+    QTest::newRow("AppImage")
+            << QFINDTESTDATA("data/hello-2.8-x86_64.AppImage")
+            << QStringLiteral("hello-2.8-x86_64")
+            << true << false << false << Archive::Unencrypted
+            << QStringLiteral("hello-2.8-x86_64");
 }
 
 void ArchiveTest::testProperties()
@@ -467,6 +477,52 @@ void ArchiveTest::testExtraction_data()
     } else {
         qDebug() << "lrzip executable not found in path. Skipping lrzip test.";
     }
+
+    // Only run test for lz4-compressed tar if lz4 executable is found in path.
+    if (!QStandardPaths::findExecutable(QStringLiteral("lz4")).isEmpty()) {
+        archivePath = QFINDTESTDATA("data/simplearchive.tar.lz4");
+        QTest::newRow("extract selected entries from a lz4-compressed tarball without path")
+                << archivePath
+                << QVariantList {
+                       QVariant::fromValue(fileRootNodePair(QStringLiteral("file3.txt"), QString())),
+                       QVariant::fromValue(fileRootNodePair(QStringLiteral("dir2/file22.txt"), QString()))
+                   }
+                << ExtractionOptions()
+                << 2;
+
+        archivePath = QFINDTESTDATA("data/simplearchive.tar.lz4");
+        QTest::newRow("extract all entries from a lz4-compressed tarball with path")
+                << archivePath
+                << QVariantList()
+                << optionsPreservePaths
+                << 7;
+    } else {
+        qDebug() << "lz4 executable not found in path. Skipping lz4 test.";
+    }
+
+    archivePath = QFINDTESTDATA("data/simplearchive.xar");
+    QTest::newRow("extract selected entries from a xar archive without path")
+            << archivePath
+            << QVariantList {
+                   QVariant::fromValue(fileRootNodePair(QStringLiteral("dir1/file11.txt"), QString())),
+                   QVariant::fromValue(fileRootNodePair(QStringLiteral("file4.txt"), QString()))
+               }
+            << ExtractionOptions()
+            << 2;
+
+    archivePath = QFINDTESTDATA("data/simplearchive.xar");
+    QTest::newRow("extract all entries from a xar archive with path")
+            << archivePath
+            << QVariantList()
+            << optionsPreservePaths
+            << 6;
+
+    archivePath = QFINDTESTDATA("data/hello-2.8-x86_64.AppImage");
+    QTest::newRow("extract all entries from an AppImage with path")
+            << archivePath
+            << QVariantList()
+            << optionsPreservePaths
+            << 7;
 }
 
 void ArchiveTest::testExtraction()
@@ -491,16 +547,12 @@ void ArchiveTest::testExtraction()
     QEventLoop eventLoop(this);
     connect(extractionJob, &KJob::result, &eventLoop, &QEventLoop::quit);
     extractionJob->start();
-    eventLoop.exec();
-
-    // We need to wait for the QTemporaryDir in CliInterface::copyFiles() to autodelete itself.
-    // TODO: find a better solution. Possibly related to task T1771 ?
-    QTest::qSleep(250);
+    eventLoop.exec(); // krazy:exclude=crashy
 
     QFETCH(int, expectedExtractedEntriesCount);
     int extractedEntriesCount = 0;
 
-    QDirIterator dirIt(destDir.path(), QDir::AllEntries | QDir::NoDotAndDotDot, QDirIterator::Subdirectories);
+    QDirIterator dirIt(destDir.path(), QDir::AllEntries | QDir::Hidden | QDir::NoDotAndDotDot, QDirIterator::Subdirectories);
     while (dirIt.hasNext()) {
         extractedEntriesCount++;
         dirIt.next();
