@@ -53,11 +53,14 @@ void MimeTypeTest::testMimeTypeDetection_data()
     const QString compressedXzTarMime = QStringLiteral("application/x-xz-compressed-tar");
     const QString compressedLzmaTarMime = QStringLiteral("application/x-lzma-compressed-tar");
     const QString compressedZTarMime = QStringLiteral("application/x-tarz");
-    const QString compressedLzipTarMime = QStringLiteral("application/x-lzip");
+    const QString compressedLzipTarMime = QStringLiteral("application/x-lzip-compressed-tar");
     const QString compressedLzopTarMime = QStringLiteral("application/x-tzo");
     const QString compressedLrzipTarMime = QStringLiteral("application/x-lrzip-compressed-tar");
+    const QString compressedLz4TarMime = QStringLiteral("application/x-lz4-compressed-tar");
     const QString isoMimeType = QStringLiteral("application/x-cd-image");
     const QString debMimeType = QMimeDatabase().mimeTypeForFile(QStringLiteral("dummy.deb"), QMimeDatabase::MatchExtension).name();
+    const QString xarMimeType = QStringLiteral("application/x-xar");
+    const QString appImageMimeType = QStringLiteral("application/x-iso9660-appimage");
 
     QTest::newRow("empty name") << QString() << QStringLiteral("application/octet-stream");
     QTest::newRow("tar.gz") << QFINDTESTDATA("data/simplearchive.tar.gz") << compressedGzipTarMime;
@@ -68,7 +71,10 @@ void MimeTypeTest::testMimeTypeDetection_data()
     QTest::newRow("tar.lz") << QFINDTESTDATA("data/simplearchive.tar.lz") << compressedLzipTarMime;
     QTest::newRow("tar.lzo") << QFINDTESTDATA("data/simplearchive.tar.lzo") << compressedLzopTarMime;
     QTest::newRow("tar.lrz") << QFINDTESTDATA("data/simplearchive.tar.lrz") << compressedLrzipTarMime;
+    QTest::newRow("tar.lz4") << QFINDTESTDATA("data/simplearchive.tar.lz4") << compressedLz4TarMime;
     QTest::newRow("deb") << QFINDTESTDATA("data/smallarchive.deb") << debMimeType;
+    QTest::newRow("xar") << QFINDTESTDATA("data/simplearchive.xar") << xarMimeType;
+    QTest::newRow("AppImage") << QFINDTESTDATA("data/hello-2.8-x86_64.AppImage") << appImageMimeType;
 
     QTest::newRow("zip with wrong extension") << QFINDTESTDATA("data/zip_with_wrong_extension.rar") << QStringLiteral("application/zip");
     QTest::newRow("tar with special char in the extension") << QStringLiteral("foo.tar~1.gz") << compressedGzipTarMime;
