@@ -30,6 +30,7 @@
 #include "archiveinterface.h"
 #include "jobs.h"
 #include "mimetypes.h"
+#include "pluginmanager.h"
 
 #include <QByteArray>
 #include <QDebug>
@@ -44,46 +45,6 @@
 namespace Kerfuffle
 {
 
-bool Archive::comparePlugins(const KPluginMetaData &p1, const KPluginMetaData &p2)
-{
-    return (p1.rawData()[QStringLiteral("X-KDE-Priority")].toVariant().toInt()) > (p2.rawData()[QStringLiteral("X-KDE-Priority")].toVariant().toInt());
-}
-
-QVector<KPluginMetaData> Archive::findPluginOffers(const QString& filename, const QString& fixedMimeType)
-{
-    qCDebug(ARK) << "Find plugin offers for" << filename << "with mime" << fixedMimeType;
-
-    const QString mimeType = fixedMimeType.isEmpty() ? determineMimeType(filename).name() : fixedMimeType;
-
-    qCDebug(ARK) << "Detected mime" << mimeType;
-
-    QVector<KPluginMetaData> offers = KPluginLoader::findPlugins(QStringLiteral("kerfuffle"), [mimeType](const KPluginMetaData& metaData) {
-        if (!metaData.serviceTypes().contains(QStringLiteral("Kerfuffle/Plugin"))) {
-            return false;
-        }
-
-        if (metaData.mimeTypes().contains(mimeType)) {
-            return true;
-        }
-
-        // The mimetype is not directly supported, but it could inherit from a supported one.
-        if (!supportedMimeTypes().contains(mimeType)) {
-            foreach (const QString &mime, metaData.mimeTypes()) {
-                if (QMimeDatabase().mimeTypeForName(mimeType).inherits(mime)) {
-                    return true;
-                }
-            }
-        }
-
-        return false;
-    });
-
-    qSort(offers.begin(), offers.end(), comparePlugins);
-    qCDebug(ARK) << "Have" << offers.size() << "offers";
-
-    return offers;
-}
-
 QDebug operator<<(QDebug d, const fileRootNodePair &pair)
 {
     d.nospace() << "fileRootNodePair(" << pair.file << "," << pair.rootNode << ")";
@@ -101,15 +62,18 @@ Archive *Archive::create(const QString &fileName, const QString &fixedMimeType,
 
     qRegisterMetaType<ArchiveEntry>("ArchiveEntry");
 
-    const QVector<KPluginMetaData> offers = findPluginOffers(fileName, fixedMimeType);
+    PluginManager pluginManager;
+    const QMimeType mimeType = fixedMimeType.isEmpty() ? determineMimeType(fileName) : QMimeDatabase().mimeTypeForName(fixedMimeType);
+
+    const QVector<Plugin*> offers = pluginManager.preferredPluginsFor(mimeType);
     if (offers.isEmpty()) {
         qCCritical(ARK) << "Could not find a plugin to handle" << fileName;
         return new Archive(NoPlugin, parent);
     }
 
     Archive *archive;
-    foreach (const KPluginMetaData& pluginMetadata, offers) {
-        archive = create(fileName, pluginMetadata, parent);
+    foreach (Plugin *plugin, offers) {
+        archive = create(fileName, plugin, parent);
         // Use the first valid plugin, according to the priority sorting.
         if (archive->isValid()) {
             return archive;
@@ -120,42 +84,32 @@ Archive *Archive::create(const QString &fileName, const QString &fixedMimeType,
     return archive;
 }
 
-Archive *Archive::create(const QString &fileName, const KPluginMetaData &pluginMetadata, QObject *parent)
+Archive *Archive::create(const QString &fileName, Plugin *plugin, QObject *parent)
 {
-    const bool isReadOnly = !pluginMetadata.rawData()[QStringLiteral("X-KDE-Kerfuffle-ReadWrite")].toVariant().toBool();
-    qCDebug(ARK) << "Loading plugin" << pluginMetadata.pluginId();
+    Q_ASSERT(plugin);
 
-    KPluginFactory *factory = KPluginLoader(pluginMetadata.fileName()).factory();
+    qCDebug(ARK) << "Checking plugin" << plugin->metaData().pluginId();
+
+    KPluginFactory *factory = KPluginLoader(plugin->metaData().fileName()).factory();
     if (!factory) {
-        qCWarning(ARK) << "Invalid plugin factory for" << pluginMetadata.pluginId();
+        qCWarning(ARK) << "Invalid plugin factory for" << plugin->metaData().pluginId();
         return new Archive(FailedPlugin, parent);
     }
 
     const QVariantList args = {QVariant(QFileInfo(fileName).absoluteFilePath())};
     ReadOnlyArchiveInterface *iface = factory->create<ReadOnlyArchiveInterface>(Q_NULLPTR, args);
     if (!iface) {
-        qCWarning(ARK) << "Could not create plugin instance" << pluginMetadata.pluginId();
+        qCWarning(ARK) << "Could not create plugin instance" << plugin->metaData().pluginId();
         return new Archive(FailedPlugin, parent);
     }
 
-    // Not CliBased plugin, don't search for executables.
-    if (!iface->isCliBased()) {
-        return new Archive(iface, isReadOnly, parent);
+    if (!plugin->isValid()) {
+        qCDebug(ARK) << "Cannot use plugin" << plugin->metaData().pluginId() << "- check whether" << plugin->readOnlyExecutables() << "are installed.";
+        return new Archive(FailedPlugin, parent);
     }
 
-    qCDebug(ARK) << "Finding executables for plugin" << pluginMetadata.pluginId();
-
-    if (iface->findExecutables(!isReadOnly)) {
-        return new Archive(iface, isReadOnly, parent);
-    }
-
-    if (!isReadOnly && iface->findExecutables(false)) {
-        qCWarning(ARK) << "Failed to find read-write executables: falling back to read-only mode for read-write plugin" << pluginMetadata.pluginId();
-        return new Archive(iface, true, parent);
-    }
-
-    qCWarning(ARK) << "Failed to find needed executables for plugin" << pluginMetadata.pluginId();
-    return new Archive(FailedPlugin, parent);
+    qCDebug(ARK) << "Successfully loaded plugin" << plugin->metaData().pluginId();
+    return new Archive(iface, !plugin->isReadWrite(), parent);
 }
 
 Archive::Archive(ArchiveError errorCode, QObject *parent)
@@ -172,10 +126,12 @@ Archive::Archive(ReadOnlyArchiveInterface *archiveInterface, bool isReadOnly, QO
         , m_hasBeenListed(false)
         , m_isReadOnly(isReadOnly)
         , m_isSingleFolderArchive(false)
+        , m_isMultiVolume(false)
         , m_extractedFilesSize(0)
         , m_error(NoError)
         , m_encryptionType(Unencrypted)
         , m_numberOfFiles(0)
+        , m_numberOfFolders(0)
 {
     qCDebug(ARK) << "Created archive instance";
 
@@ -215,9 +171,41 @@ QString Archive::comment() const
     return isValid() ? m_iface->comment() : QString();
 }
 
-QMimeType Archive::mimeType() const
+CommentJob* Archive::addComment(const QString &comment)
 {
-    return isValid() ? determineMimeType(fileName()) : QMimeType();
+    if (!isValid()) {
+        return Q_NULLPTR;
+    }
+
+    qCDebug(ARK) << "Going to add comment:" << comment;
+    Q_ASSERT(!isReadOnly());
+    CommentJob *job = new CommentJob(comment, static_cast<ReadWriteArchiveInterface*>(m_iface));
+    return job;
+}
+
+TestJob* Archive::testArchive()
+{
+    if (!isValid()) {
+        return Q_NULLPTR;
+    }
+
+    qCDebug(ARK) << "Going to test archive";
+
+    TestJob *job = new TestJob(m_iface);
+    return job;
+}
+
+QMimeType Archive::mimeType()
+{
+    if (!isValid()) {
+        return QMimeType();
+    }
+
+    if (!m_mimeType.isValid()) {
+        m_mimeType = determineMimeType(fileName());
+    }
+
+    return m_mimeType;
 }
 
 bool Archive::isReadOnly() const
@@ -240,6 +228,21 @@ bool Archive::hasComment() const
     return isValid() ? !comment().isEmpty() : false;
 }
 
+bool Archive::isMultiVolume() const
+{
+    return m_iface->isMultiVolume();
+}
+
+void Archive::setMultiVolume(bool value)
+{
+    m_iface->setMultiVolume(value);
+}
+
+int Archive::numberOfVolumes() const
+{
+    return m_iface->numberOfVolumes();
+}
+
 Archive::EncryptionType Archive::encryptionType()
 {
     if (!isValid()) {
@@ -250,6 +253,11 @@ Archive::EncryptionType Archive::encryptionType()
     return m_encryptionType;
 }
 
+QString Archive::password() const
+{
+    return m_iface->password();
+}
+
 qulonglong Archive::numberOfFiles()
 {
     if (!isValid()) {
@@ -260,6 +268,16 @@ qulonglong Archive::numberOfFiles()
     return m_numberOfFiles;
 }
 
+qulonglong Archive::numberOfFolders()
+{
+    if (!isValid()) {
+        return 0;
+    }
+
+    listIfNotListed();
+    return m_numberOfFolders;
+}
+
 qulonglong Archive::unpackedSize()
 {
     if (!isValid()) {
@@ -287,9 +305,7 @@ QString Archive::subfolderName()
 
 void Archive::onNewEntry(const ArchiveEntry &entry)
 {
-    if (!entry[IsDirectory].toBool()) {
-        m_numberOfFiles++;
-    }
+    entry[IsDirectory].toBool() ? m_numberOfFolders++ : m_numberOfFiles++;
 }
 
 bool Archive::isValid() const
@@ -320,7 +336,7 @@ ListJob* Archive::list()
 
     qCDebug(ARK) << "Going to list files";
 
-    ListJob *job = new ListJob(m_iface, this);
+    ListJob *job = new ListJob(m_iface);
 
     //if this job has not been listed before, we grab the opportunity to
     //collect some information about the archive
@@ -341,7 +357,7 @@ DeleteJob* Archive::deleteFiles(const QList<QVariant> & files)
     if (m_iface->isReadOnly()) {
         return 0;
     }
-    DeleteJob *newJob = new DeleteJob(files, static_cast<ReadWriteArchiveInterface*>(m_iface), this);
+    DeleteJob *newJob = new DeleteJob(files, static_cast<ReadWriteArchiveInterface*>(m_iface));
 
     return newJob;
 }
@@ -352,9 +368,15 @@ AddJob* Archive::addFiles(const QStringList & files, const CompressionOptions& o
         return Q_NULLPTR;
     }
 
-    qCDebug(ARK) << "Going to add files" << files << "with options" << options;
+    CompressionOptions newOptions = options;
+    if (encryptionType() != Unencrypted) {
+        newOptions[QStringLiteral("PasswordProtectedHint")] = true;
+    }
+
+    qCDebug(ARK) << "Going to add files" << files << "with options" << newOptions;
     Q_ASSERT(!m_iface->isReadOnly());
-    AddJob *newJob = new AddJob(files, options, static_cast<ReadWriteArchiveInterface*>(m_iface), this);
+
+    AddJob *newJob = new AddJob(files, newOptions, static_cast<ReadWriteArchiveInterface*>(m_iface));
     connect(newJob, &AddJob::result, this, &Archive::onAddFinished);
     return newJob;
 }
@@ -370,10 +392,40 @@ ExtractJob* Archive::copyFiles(const QList<QVariant>& files, const QString& dest
         newOptions[QStringLiteral( "PasswordProtectedHint" )] = true;
     }
 
-    ExtractJob *newJob = new ExtractJob(files, destinationDir, newOptions, m_iface, this);
+    ExtractJob *newJob = new ExtractJob(files, destinationDir, newOptions, m_iface);
     return newJob;
 }
 
+PreviewJob *Archive::preview(const QString &file)
+{
+    if (!isValid()) {
+        return Q_NULLPTR;
+    }
+
+    PreviewJob *job = new PreviewJob(file, (encryptionType() != Unencrypted), m_iface);
+    return job;
+}
+
+OpenJob *Archive::open(const QString &file)
+{
+    if (!isValid()) {
+        return Q_NULLPTR;
+    }
+
+    OpenJob *job = new OpenJob(file, (encryptionType() != Unencrypted), m_iface);
+    return job;
+}
+
+OpenWithJob *Archive::openWith(const QString &file)
+{
+    if (!isValid()) {
+        return Q_NULLPTR;
+    }
+
+    OpenWithJob *job = new OpenWithJob(file, (encryptionType() != Unencrypted), m_iface);
+    return job;
+}
+
 void Archive::encrypt(const QString &password, bool encryptHeader)
 {
     if (!isValid()) {
@@ -439,4 +491,19 @@ void Archive::onUserQuery(Query* query)
     query->execute();
 }
 
+void Archive::setCompressionOptions(const CompressionOptions &opts)
+{
+    m_compOptions = opts;
+}
+
+CompressionOptions Archive::compressionOptions() const
+{
+    return m_compOptions;
+}
+
+QString Archive::multiVolumeName() const
+{
+    return m_iface->multiVolumeName();
+}
+
 } // namespace Kerfuffle
