@@ -45,6 +45,12 @@ class ListJob;
 class ExtractJob;
 class DeleteJob;
 class AddJob;
+class CommentJob;
+class TestJob;
+class OpenJob;
+class OpenWithJob;
+class Plugin;
+class PreviewJob;
 class Query;
 class ReadOnlyArchiveInterface;
 
@@ -145,11 +151,15 @@ class KERFUFFLE_EXPORT Archive : public QObject
     Q_PROPERTY(QMimeType mimeType READ mimeType CONSTANT)
     Q_PROPERTY(bool isReadOnly READ isReadOnly CONSTANT)
     Q_PROPERTY(bool isSingleFolderArchive READ isSingleFolderArchive)
+    Q_PROPERTY(bool isMultiVolume READ isMultiVolume WRITE setMultiVolume)
+    Q_PROPERTY(bool numberOfVolumes READ numberOfVolumes)
     Q_PROPERTY(EncryptionType encryptionType READ encryptionType)
     Q_PROPERTY(qulonglong numberOfFiles READ numberOfFiles)
+    Q_PROPERTY(qulonglong numberOfFolders READ numberOfFolders)
     Q_PROPERTY(qulonglong unpackedSize READ unpackedSize)
     Q_PROPERTY(qulonglong packedSize READ packedSize)
     Q_PROPERTY(QString subfolderName READ subfolderName)
+    Q_PROPERTY(QString password READ password)
 
 public:
 
@@ -163,29 +173,33 @@ public:
     QString completeBaseName() const;
     QString fileName() const;
     QString comment() const;
-    QMimeType mimeType() const;
+    QMimeType mimeType();
     bool isReadOnly() const;
     bool isSingleFolderArchive();
+    bool isMultiVolume() const;
+    void setMultiVolume(bool value);
     bool hasComment() const;
+    int numberOfVolumes() const;
     EncryptionType encryptionType();
+    QString password() const;
     qulonglong numberOfFiles();
+    qulonglong numberOfFolders();
     qulonglong unpackedSize();
     qulonglong packedSize() const;
     QString subfolderName();
-
-    static bool comparePlugins(const KPluginMetaData &p1, const KPluginMetaData &p2);
-    static QVector<KPluginMetaData> findPluginOffers(const QString& filename, const QString& fixedMimeType);
+    void setCompressionOptions(const CompressionOptions &opts);
+    CompressionOptions compressionOptions() const;
+    QString multiVolumeName() const;
 
     static Archive *create(const QString &fileName, QObject *parent = 0);
     static Archive *create(const QString &fileName, const QString &fixedMimeType, QObject *parent = 0);
 
     /**
-     * Create an archive instance from a given plugin.
+     * Create an archive instance from a given @p plugin.
      * @param fileName The name of the archive.
-     * @param pluginMetadata The plugin's metadata.
      * @return A valid archive if the plugin could be loaded, an invalid one otherwise (with the FailedPlugin error set).
      */
-    static Archive *create(const QString &fileName, const KPluginMetaData &pluginMetadata, QObject *parent = Q_NULLPTR);
+    static Archive *create(const QString &fileName, Plugin *plugin, QObject *parent = Q_NULLPTR);
 
     ~Archive();
 
@@ -201,6 +215,8 @@ public:
     ListJob* list();
 
     DeleteJob* deleteFiles(const QList<QVariant> & files);
+    CommentJob* addComment(const QString &comment);
+    TestJob* testArchive();
 
     /**
      * Compression options that should be handled by all interfaces:
@@ -218,6 +234,10 @@ public:
 
     ExtractJob* copyFiles(const QList<QVariant> &files, const QString &destinationDir, const ExtractionOptions &options = ExtractionOptions());
 
+    PreviewJob* preview(const QString &file);
+    OpenJob* open(const QString &file);
+    OpenWithJob* openWith(const QString &file);
+
     /**
      * @param password The password to encrypt the archive with.
      * @param encryptHeader Whether to encrypt also the list of files.
@@ -239,12 +259,16 @@ private:
     bool m_hasBeenListed;
     bool m_isReadOnly;
     bool m_isSingleFolderArchive;
+    bool m_isMultiVolume;
 
     QString m_subfolderName;
     qulonglong m_extractedFilesSize;
     ArchiveError m_error;
     EncryptionType m_encryptionType;
     qulonglong m_numberOfFiles;
+    qulonglong m_numberOfFolders;
+    CompressionOptions m_compOptions;
+    QMimeType m_mimeType;
 };
 
 } // namespace Kerfuffle
