@@ -46,10 +46,12 @@
 #include <QDirIterator>
 #include <QEventLoop>
 #include <QFile>
+#include <QMimeDatabase>
 #include <QProcess>
 #include <QRegularExpression>
 #include <QStandardPaths>
 #include <QTemporaryDir>
+#include <QTemporaryFile>
 #include <QThread>
 #include <QTimer>
 #include <QUrl>
@@ -60,7 +62,9 @@ CliInterface::CliInterface(QObject *parent, const QVariantList & args)
         : ReadWriteArchiveInterface(parent, args),
         m_process(0),
         m_listEmptyLines(false),
-        m_abortingOperation(false)
+        m_abortingOperation(false),
+        m_extractTempDir(Q_NULLPTR),
+        m_commentTempFile(Q_NULLPTR)
 {
     //because this interface uses the event loop
     setWaitForFinishedSignal(true);
@@ -83,39 +87,9 @@ void CliInterface::cacheParameterList()
 CliInterface::~CliInterface()
 {
     Q_ASSERT(!m_process);
+    delete m_commentTempFile;
 }
 
-bool CliInterface::isCliBased() const
-{
-    return true;
-}
-
-bool CliInterface::findExecutables(bool isReadWrite)
-{
-    cacheParameterList();
-
-    QList<int> execTypes = QList<int>() << ListProgram << ExtractProgram;
-    if (isReadWrite) {
-        execTypes << AddProgram << DeleteProgram;
-    }
-
-    foreach (const int execType, execTypes) {
-        bool execTypeFound = false;
-        foreach (const QString &program, m_param.value(execType).toStringList()) {
-            if (!QStandardPaths::findExecutable(program).isEmpty()) {
-                qCDebug(ARK) << "Found executable type" << execType << ":" << program;
-                execTypeFound = true;
-                break;
-            }
-        }
-        if (!execTypeFound) {
-            return false;
-        }
-    }
-    return true;
-}
-
-
 void CliInterface::setListEmptyLines(bool emptyLines)
 {
     m_listEmptyLines = emptyLines;
@@ -127,15 +101,12 @@ bool CliInterface::list()
     cacheParameterList();
     m_operationMode = List;
 
-    QStringList args = m_param.value(ListArgs).toStringList();
-    substituteListVariables(args);
+    const auto args = substituteListVariables(m_param.value(ListArgs).toStringList(), password());
 
     if (!runProcess(m_param.value(ListProgram).toStringList(), args)) {
-        failOperation();
         return false;
     }
 
-    emit finished(true);
     return true;
 }
 
@@ -145,6 +116,9 @@ bool CliInterface::copyFiles(const QVariantList &files, const QString &destinati
 
     cacheParameterList();
     m_operationMode = Copy;
+    m_compressionOptions = options;
+    m_copiedFiles = files;
+    m_extractDestDir = destinationDirectory;
     const QStringList extractArgs = m_param.value(ExtractArgs).toStringList();
 
     if (extractArgs.contains(QStringLiteral("$PasswordSwitch")) &&
@@ -160,83 +134,34 @@ bool CliInterface::copyFiles(const QVariantList &files, const QString &destinati
     const QStringList args = substituteCopyVariables(extractArgs,
                                                      files,
                                                      options.value(QStringLiteral("PreservePaths")).toBool(),
-                                                     password(),
-                                                     options.value(QStringLiteral("RootNode"), QString()).toString());
+                                                     password());
 
     QUrl destDir = QUrl(destinationDirectory);
     QDir::setCurrent(destDir.adjusted(QUrl::RemoveScheme).url());
 
-    QString oldCurrentDir;
     bool useTmpExtractDir = options.value(QStringLiteral("DragAndDrop")).toBool() ||
                             options.value(QStringLiteral("AlwaysUseTmpDir")).toBool();
-    QTemporaryDir tmpExtractDir(QApplication::applicationName() + QLatin1Char('-'));
 
     if (useTmpExtractDir) {
-        qCDebug(ARK) << "Using temporary extraction dir:" << tmpExtractDir.path();
-        if (!tmpExtractDir.isValid()) {
+
+        Q_ASSERT(!m_extractTempDir);
+        m_extractTempDir = new QTemporaryDir(QApplication::applicationName() + QLatin1Char('-'));
+
+        qCDebug(ARK) << "Using temporary extraction dir:" << m_extractTempDir->path();
+        if (!m_extractTempDir->isValid()) {
             qCDebug(ARK) << "Creation of temporary directory failed.";
-            failOperation();
+            emit finished(false);
             return false;
         }
-        oldCurrentDir = QDir::currentPath();
-        destDir = QUrl(tmpExtractDir.path());
+        m_oldWorkingDir = QDir::currentPath();
+        destDir = QUrl(m_extractTempDir->path());
         QDir::setCurrent(destDir.adjusted(QUrl::RemoveScheme).url());
     }
 
     if (!runProcess(m_param.value(ExtractProgram).toStringList(), args)) {
-        failOperation();
         return false;
     }
 
-    if (options.value(QStringLiteral("AlwaysUseTmpDir")).toBool()) {
-        // unar exits with code 1 if extraction fails.
-        // This happens at least with wrong passwords or not enough space in the destination folder.
-        if (m_exitCode == 1) {
-            if (password().isEmpty()) {
-                qCWarning(ARK) << "Extraction aborted, destination folder might not have enough space.";
-                emit error(i18n("Extraction failed. Make sure that enough space is available."));
-            } else {
-                qCWarning(ARK) << "Extraction aborted, either the password is wrong or the destination folder doesn't have enough space.";
-                emit error(i18n("Extraction failed. Make sure you provided the correct password and that enough space is available."));
-                setPassword(QString());
-            }
-
-            emit finished(false);
-            failOperation();
-            // If we don't do this, the temporary directory will not autodelete itself upon destruction.
-            QDir::setCurrent(oldCurrentDir);
-
-            return false;
-        }
-
-        if (!options.value(QStringLiteral("DragAndDrop")).toBool()) {
-            if (!moveToDestination(QDir::current(), QDir(destinationDirectory), options[QStringLiteral("PreservePaths")].toBool())) {
-                emit error(i18ncp("@info",
-                                  "Could not move the extracted file to the destination directory.",
-                                  "Could not move the extracted files to the destination directory.",
-                                  files.size()));
-                emit finished(false);
-                return false;
-            }
-            // If we don't do this, the temporary directory will not autodelete itself upon destruction.
-            QDir::setCurrent(oldCurrentDir);
-        }
-    }
-
-    if (options.value(QStringLiteral("DragAndDrop")).toBool()) {
-        if (!moveDroppedFilesToDest(files, destinationDirectory)) {
-            emit error(i18ncp("@info",
-                              "Could not move the extracted file to the destination directory.",
-                              "Could not move the extracted files to the destination directory.",
-                              files.size()));
-            emit finished(false);
-            return false;
-        }
-        // If we don't do this, the temporary directory will not autodelete itself upon destruction.
-        QDir::setCurrent(oldCurrentDir);
-    }
-
-    emit finished(true);
     return true;
 }
 
@@ -246,106 +171,31 @@ bool CliInterface::addFiles(const QStringList & files, const CompressionOptions&
 
     m_operationMode = Add;
 
-    const QString globalWorkDir = options.value(QStringLiteral( "GlobalWorkDir" )).toString();
-    const QDir workDir = globalWorkDir.isEmpty() ? QDir::current() : QDir(globalWorkDir);
-    if (!globalWorkDir.isEmpty()) {
-        qCDebug(ARK) << "GlobalWorkDir is set, changing dir to " << globalWorkDir;
-        QDir::setCurrent(globalWorkDir);
-    }
+    const QStringList addArgs = m_param.value(AddArgs).toStringList();
 
-    //start preparing the argument list
-    QStringList args = m_param.value(AddArgs).toStringList();
-
-    //now replace the various elements in the list
-    for (int i = 0; i < args.size(); ++i) {
-        const QString argument = args.at(i);
-        qCDebug(ARK) << "Processing argument " << argument;
-
-        if (argument == QLatin1String( "$Archive" )) {
-            args[i] = filename();
-        }
-
-        if (argument == QLatin1String("$PasswordSwitch")) {
-            //if the PasswordSwitch argument has been added, we at least
-            //assume that the format of the switch has been added as well
-            Q_ASSERT(m_param.contains(PasswordSwitch));
-
-            //we will decrement i afterwards
-            args.removeAt(i);
-
-            QString pass = password();
-
-            if (!pass.isEmpty()) {
-                QStringList theSwitch = m_param.value(PasswordSwitch).toStringList();
-
-                // use the header encryption switch if needed and if provided by the plugin
-                if (isHeaderEncryptionEnabled() && !m_param.value(PasswordHeaderSwitch).toStringList().isEmpty()) {
-                    theSwitch = m_param.value(PasswordHeaderSwitch).toStringList();
-                }
-
-                for (int j = 0; j < theSwitch.size(); ++j) {
-                    //get the argument part
-                    QString newArg = theSwitch.at(j);
-
-                    //substitute the $Password
-                    newArg.replace(QLatin1String("$Password"), pass);
-
-                    //put it in the arg list
-                    args.insert(i + j, newArg);
-                    ++i;
-                }
-            }
-            --i; //decrement to compensate for the variable we replaced
-        }
-
-        if (argument == QLatin1String("$EncryptHeaderSwitch")) {
-            //if the EncryptHeaderSwitch argument has been added, we at least
-            //assume that the format of the switch has been added as well
-            Q_ASSERT(m_param.contains(EncryptHeaderSwitch));
-
-            //we will decrement i afterwards
-            args.removeAt(i);
-
-            QString enabled = isHeaderEncryptionEnabled() ? QStringLiteral("-mhe=on") : QString();
-
-            QStringList theSwitch = m_param.value(EncryptHeaderSwitch).toStringList();
-            for (int j = 0; j < theSwitch.size(); ++j) {
-                //get the argument part
-                QString newArg = theSwitch.at(j);
-
-                //substitute the $Password
-                newArg.replace(QLatin1String("$Enabled"), enabled);
-
-                //put it in the arg list
-                args.insert(i + j, newArg);
-                ++i;
-            }
-
-            --i; //decrement to compensate for the variable we replaced
-        }
-
-        if (argument == QLatin1String( "$Files" )) {
-            args.removeAt(i);
-            for (int j = 0; j < files.count(); ++j) {
-                // #191821: workDir must be used instead of QDir::current()
-                //          so that symlinks aren't resolved automatically
-                // TODO: this kind of call should be moved upwards in the
-                //       class hierarchy to avoid code duplication
-                const QString relativeName =
-                    workDir.relativeFilePath(files.at(j));
-
-                args.insert(i + j, relativeName);
-                ++i;
-            }
-            --i;
+    if (addArgs.contains(QStringLiteral("$PasswordSwitch")) &&
+        options.value(QStringLiteral("PasswordProtectedHint")).toBool() &&
+        password().isEmpty()) {
+        qCDebug(ARK) << "Password hint enabled, querying user";
+        if (!passwordQuery()) {
+            return false;
         }
     }
 
+    int compLevel = options.value(QStringLiteral("CompressionLevel"), -1).toInt();
+    ulong volumeSize = options.value(QStringLiteral("VolumeSize"), 0).toULongLong();
+
+    const auto args = substituteAddVariables(m_param.value(AddArgs).toStringList(),
+                                             files,
+                                             password(),
+                                             isHeaderEncryptionEnabled(),
+                                             compLevel,
+                                             volumeSize);
+
     if (!runProcess(m_param.value(AddProgram).toStringList(), args)) {
-        failOperation();
         return false;
     }
-    emit finished(true);
+
     return true;
 }
 
@@ -363,15 +213,31 @@ bool CliInterface::deleteFiles(const QList<QVariant> & files)
                                                 password());
 
     if (!runProcess(m_param.value(DeleteProgram).toStringList(), args)) {
-        failOperation();
         return false;
     }
-    emit finished(true);
+
+    return true;
+}
+
+bool CliInterface::testArchive()
+{
+    resetParsing();
+    cacheParameterList();
+    m_operationMode = Test;
+
+    const auto args = substituteTestVariables(m_param.value(TestArgs).toStringList(), password());
+
+    if (!runProcess(m_param.value(TestProgram).toStringList(), args)) {
+        return false;
+    }
+
     return true;
 }
 
 bool CliInterface::runProcess(const QStringList& programNames, const QStringList& arguments)
 {
+    Q_ASSERT(!m_process);
+
     QString programPath;
     for (int i = 0; i < programNames.count(); i++) {
         programPath = QStandardPaths::findExecutable(programNames.at(i));
@@ -388,18 +254,11 @@ bool CliInterface::runProcess(const QStringList& programNames, const QStringList
 
     qCDebug(ARK) << "Executing" << programPath << arguments << "within directory" << QDir::currentPath();
 
-    if (m_process) {
-        m_process->waitForFinished();
-        delete m_process;
-    }
-
 #ifdef Q_OS_WIN
     m_process = new KProcess;
 #else
     m_process = new KPtyProcess;
     m_process->setPtyChannels(KPtyProcess::StdinChannel);
-    QEventLoop loop;
-    connect(m_process, static_cast<void (KPtyProcess::*)(int, QProcess::ExitStatus)>(&KPtyProcess::finished), &loop, &QEventLoop::quit, Qt::DirectConnection);
 #endif
 
     m_process->setOutputChannelMode(KProcess::MergedChannels);
@@ -407,21 +266,19 @@ bool CliInterface::runProcess(const QStringList& programNames, const QStringList
     m_process->setProgram(programPath, arguments);
 
     connect(m_process, SIGNAL(readyReadStandardOutput()), SLOT(readStdout()), Qt::DirectConnection);
-    connect(m_process, static_cast<void (KPtyProcess::*)(int, QProcess::ExitStatus)>(&KPtyProcess::finished), this, &CliInterface::processFinished, Qt::DirectConnection);
+
+    if (m_operationMode == Copy) {
+        // Extraction jobs need a dedicated post-processing function.
+        connect(m_process, static_cast<void (KPtyProcess::*)(int, QProcess::ExitStatus)>(&KPtyProcess::finished), this, &CliInterface::copyProcessFinished, Qt::DirectConnection);
+    } else {
+        connect(m_process, static_cast<void (KPtyProcess::*)(int, QProcess::ExitStatus)>(&KPtyProcess::finished), this, &CliInterface::processFinished, Qt::DirectConnection);
+    }
 
     m_stdOutData.clear();
 
     m_process->start();
 
-#ifdef Q_OS_WIN
-    bool ret = m_process->waitForFinished(-1);
-#else
-    bool ret = (loop.exec(QEventLoop::WaitForMoreEvents | QEventLoop::ExcludeUserInputEvents) == 0);
-#endif
-
-    Q_ASSERT(!m_process);
-
-    return ret;
+    return true;
 }
 
 void CliInterface::processFinished(int exitCode, QProcess::ExitStatus exitStatus)
@@ -429,9 +286,17 @@ void CliInterface::processFinished(int exitCode, QProcess::ExitStatus exitStatus
     m_exitCode = exitCode;
     qCDebug(ARK) << "Process finished, exitcode:" << exitCode << "exitstatus:" << exitStatus;
 
-    //if the m_process pointer is gone, then there is nothing to worry
-    //about here
-    if (!m_process) {
+    if (m_process) {
+        //handle all the remaining data in the process
+        readStdout(true);
+
+        delete m_process;
+        m_process = Q_NULLPTR;
+    }
+
+    // #193908 - #222392
+    // Don't emit finished() if the job was killed quietly.
+    if (m_abortingOperation) {
         return;
     }
 
@@ -441,18 +306,88 @@ void CliInterface::processFinished(int exitCode, QProcess::ExitStatus exitStatus
         }
     }
 
-    //handle all the remaining data in the process
-    readStdout(true);
+    if (m_operationMode == Add && !isMultiVolume()) {
+        list();
+    } else if (m_operationMode == List && isCorrupt()) {
+        Kerfuffle::LoadCorruptQuery query(filename());
+        emit userQuery(&query);
+        query.waitForResponse();
+        if (!query.responseYes()) {
+            emit cancelled();
+            emit finished(false);
+        } else {
+            emit progress(1.0);
+            emit finished(true);
+        }
+    } else  {
+        emit progress(1.0);
+        emit finished(true);
+    }
+}
 
-    delete m_process;
-    m_process = 0;
+void CliInterface::copyProcessFinished(int exitCode, QProcess::ExitStatus exitStatus)
+{
+    Q_ASSERT(m_operationMode == Copy);
+
+    m_exitCode = exitCode;
+    qCDebug(ARK) << "Extraction process finished, exitcode:" << exitCode << "exitstatus:" << exitStatus;
+
+    if (m_process) {
+        // Handle all the remaining data in the process.
+        readStdout(true);
+
+        delete m_process;
+        m_process = Q_NULLPTR;
+    }
+
+    if (m_compressionOptions.value(QStringLiteral("AlwaysUseTmpDir")).toBool()) {
+        // unar exits with code 1 if extraction fails.
+        // This happens at least with wrong passwords or not enough space in the destination folder.
+        if (m_exitCode == 1) {
+            if (password().isEmpty()) {
+                qCWarning(ARK) << "Extraction aborted, destination folder might not have enough space.";
+                emit error(i18n("Extraction failed. Make sure that enough space is available."));
+            } else {
+                qCWarning(ARK) << "Extraction aborted, either the password is wrong or the destination folder doesn't have enough space.";
+                emit error(i18n("Extraction failed. Make sure you provided the correct password and that enough space is available."));
+                setPassword(QString());
+            }
+            copyProcessCleanup();
+            emit finished(false);
+            return;
+        }
+
+        if (!m_compressionOptions.value(QStringLiteral("DragAndDrop")).toBool()) {
+            if (!moveToDestination(QDir::current(), QDir(m_extractDestDir), m_compressionOptions[QStringLiteral("PreservePaths")].toBool())) {
+                emit error(i18ncp("@info",
+                                  "Could not move the extracted file to the destination directory.",
+                                  "Could not move the extracted files to the destination directory.",
+                                  m_copiedFiles.size()));
+                copyProcessCleanup();
+                emit finished(false);
+                return;
+            }
+
+            copyProcessCleanup();
+        }
+    }
+
+    if (m_compressionOptions.value(QStringLiteral("DragAndDrop")).toBool()) {
+        if (!moveDroppedFilesToDest(m_copiedFiles, m_extractDestDir)) {
+            emit error(i18ncp("@info",
+                              "Could not move the extracted file to the destination directory.",
+                              "Could not move the extracted files to the destination directory.",
+                              m_copiedFiles.size()));
+            copyProcessCleanup();
+            emit finished(false);
+            return;
+        }
+
+        copyProcessCleanup();
+    }
 
     emit progress(1.0);
-
-    if (m_operationMode == Add) {
-        list();
-        return;
-    }
+    emit finished(true);
 }
 
 bool CliInterface::moveDroppedFilesToDest(const QVariantList &files, const QString &finalDest)
@@ -545,6 +480,18 @@ bool CliInterface::isEmptyDir(const QDir &dir)
     return d.count() == 0;
 }
 
+void CliInterface::copyProcessCleanup()
+{
+    if (!m_oldWorkingDir.isEmpty()) {
+        QDir::setCurrent(m_oldWorkingDir);
+    }
+
+    if (m_extractTempDir) {
+        delete m_extractTempDir;
+        m_extractTempDir = Q_NULLPTR;
+    }
+}
+
 bool CliInterface::moveToDestination(const QDir &tempDir, const QDir &destDir, bool preservePaths)
 {
     qCDebug(ARK) << "Moving extracted files from temp dir" << tempDir.path() << "to final destination" << destDir.path();
@@ -625,7 +572,34 @@ bool CliInterface::moveToDestination(const QDir &tempDir, const QDir &destDir, b
     return true;
 }
 
-QStringList CliInterface::substituteCopyVariables(const QStringList &extractArgs, const QVariantList &files, bool preservePaths, const QString &password, const QString &rootNode)
+QStringList CliInterface::substituteListVariables(const QStringList &listArgs, const QString &password)
+{
+    // Required if we call this function from unit tests.
+    cacheParameterList();
+
+    QStringList args;
+    foreach (const QString& arg, listArgs) {
+        if (arg == QLatin1String("$Archive")) {
+            args << filename();
+            continue;
+        }
+
+        if (arg == QLatin1String("$PasswordSwitch")) {
+            args << passwordSwitch(password);
+            continue;
+        }
+
+        // Simple argument (e.g. -slt in 7z), nothing to substitute, just add it to the list.
+        args << arg;
+    }
+
+    // Remove empty strings, if any.
+    args.removeAll(QString());
+
+    return args;
+}
+
+QStringList CliInterface::substituteCopyVariables(const QStringList &extractArgs, const QVariantList &files, bool preservePaths, const QString &password)
 {
     // Required if we call this function from unit tests.
     cacheParameterList();
@@ -645,16 +619,10 @@ QStringList CliInterface::substituteCopyVariables(const QStringList &extractArgs
         }
 
         if (arg == QLatin1String("$PasswordSwitch")) {
-
             args << passwordSwitch(password);
             continue;
         }
 
-        if (arg == QLatin1String("$RootNodeSwitch")) {
-            args << rootNodeSwitch(rootNode);
-            continue;
-        }
-
         if (arg == QLatin1String("$Files")) {
             args << copyFilesList(files);
             continue;
@@ -670,6 +638,50 @@ QStringList CliInterface::substituteCopyVariables(const QStringList &extractArgs
     return args;
 }
 
+QStringList CliInterface::substituteAddVariables(const QStringList &addArgs, const QStringList &files, const QString &password, bool encryptHeader, int compLevel, ulong volumeSize)
+{
+    // Required if we call this function from unit tests.
+    cacheParameterList();
+
+    QStringList args;
+    foreach (const QString& arg, addArgs) {
+        qCDebug(ARK) << "Processing argument " << arg;
+
+        if (arg == QLatin1String("$Archive")) {
+            args << filename();
+            continue;
+        }
+
+        if (arg == QLatin1String("$PasswordSwitch")) {
+            args << (encryptHeader ? passwordHeaderSwitch(password) : passwordSwitch(password));
+            continue;
+        }
+
+        if (arg == QLatin1String("$CompressionLevelSwitch")) {
+            args << compressionLevelSwitch(compLevel);
+            continue;
+        }
+
+        if (arg == QLatin1String("$MultiVolumeSwitch")) {
+            args << multiVolumeSwitch(volumeSize);
+            continue;
+        }
+
+        if (arg == QLatin1String("$Files")) {
+            args << files;
+            continue;
+        }
+
+        // Simple argument (e.g. a in 7z), nothing to substitute, just add it to the list.
+        args << arg;
+    }
+
+    // Remove empty strings, if any.
+    args.removeAll(QString());
+
+    return args;
+}
+
 QStringList CliInterface::substituteDeleteVariables(const QStringList &deleteArgs, const QVariantList &files, const QString &password)
 {
     cacheParameterList();
@@ -705,6 +717,64 @@ QStringList CliInterface::substituteDeleteVariables(const QStringList &deleteArg
     return args;
 }
 
+QStringList CliInterface::substituteCommentVariables(const QStringList &commentArgs, const QString &commentFile)
+{
+    // Required if we call this function from unit tests.
+    cacheParameterList();
+
+    QStringList args;
+    foreach (const QString& arg, commentArgs) {
+        qCDebug(ARK) << "Processing argument " << arg;
+
+        if (arg == QLatin1String("$Archive")) {
+            args << filename();
+            continue;
+        }
+
+        if (arg == QLatin1String("$CommentSwitch")) {
+            QString commentSwitch = m_param.value(CommentSwitch).toString();
+            commentSwitch.replace(QStringLiteral("$CommentFile"), commentFile);
+            args << commentSwitch;
+            continue;
+        }
+
+        args << arg;
+    }
+
+    // Remove empty strings, if any.
+    args.removeAll(QString());
+
+    return args;
+}
+
+QStringList CliInterface::substituteTestVariables(const QStringList &testArgs, const QString &password)
+{
+    // Required if we call this function from unit tests.
+    cacheParameterList();
+
+    QStringList args;
+    foreach (const QString& arg, testArgs) {
+        qCDebug(ARK) << "Processing argument " << arg;
+
+        if (arg == QLatin1String("$Archive")) {
+            args << filename();
+            continue;
+        }
+
+        if (arg == QLatin1String("$PasswordSwitch")) {
+            args << passwordSwitch(password);
+            continue;
+        }
+
+        args << arg;
+    }
+
+    // Remove empty strings, if any.
+    args.removeAll(QString());
+
+    return args;
+}
+
 QString CliInterface::preservePathSwitch(bool preservePaths) const
 {
     Q_ASSERT(m_param.contains(PreservePathSwitch));
@@ -714,6 +784,26 @@ QString CliInterface::preservePathSwitch(bool preservePaths) const
     return (preservePaths ? theSwitch.at(0) : theSwitch.at(1));
 }
 
+QStringList CliInterface::passwordHeaderSwitch(const QString& password) const
+{
+    if (password.isEmpty()) {
+        return QStringList();
+    }
+
+    Q_ASSERT(m_param.contains(PasswordHeaderSwitch));
+
+    QStringList passwordHeaderSwitch = m_param.value(PasswordHeaderSwitch).toStringList();
+    Q_ASSERT(!passwordHeaderSwitch.isEmpty() && passwordHeaderSwitch.size() <= 2);
+
+    if (passwordHeaderSwitch.size() == 1) {
+        passwordHeaderSwitch[0].replace(QLatin1String("$Password"), password);
+    } else {
+        passwordHeaderSwitch[1] = password;
+    }
+
+    return passwordHeaderSwitch;
+}
+
 QStringList CliInterface::passwordSwitch(const QString& password) const
 {
     if (password.isEmpty()) {
@@ -734,24 +824,38 @@ QStringList CliInterface::passwordSwitch(const QString& password) const
     return passwordSwitch;
 }
 
-QStringList CliInterface::rootNodeSwitch(const QString &rootNode) const
+QString CliInterface::compressionLevelSwitch(int level) const
 {
-    if (rootNode.isEmpty()) {
-        return QStringList();
+    if (level < 0 || level > 9) {
+        return QString();
     }
 
-    Q_ASSERT(m_param.contains(RootNodeSwitch));
+    Q_ASSERT(m_param.contains(CompressionLevelSwitch));
 
-    QStringList rootNodeSwitch = m_param.value(RootNodeSwitch).toStringList();
-    Q_ASSERT(!rootNodeSwitch.isEmpty() && rootNodeSwitch.size() <= 2);
+    QString compLevelSwitch = m_param.value(CompressionLevelSwitch).toString();
+    Q_ASSERT(!compLevelSwitch.isEmpty());
 
-    if (rootNodeSwitch.size() == 1) {
-        rootNodeSwitch[0].replace(QLatin1String("$Path"), rootNode);
-    } else {
-        rootNodeSwitch[1] = rootNode;
+    compLevelSwitch.replace(QLatin1String("$CompressionLevel"), QString::number(level));
+
+    return compLevelSwitch;
+}
+
+QString CliInterface::multiVolumeSwitch(ulong volumeSize) const
+{
+    // The maximum value we allow in the QDoubleSpinBox is 1000MB. Converted to
+    // KB this is 1024000.
+    if (volumeSize <= 0 || volumeSize > 1024000) {
+        return QString();
     }
 
-    return rootNodeSwitch;
+    Q_ASSERT(m_param.contains(MultiVolumeSwitch));
+
+    QString multiVolumeSwitch = m_param.value(MultiVolumeSwitch).toString();
+    Q_ASSERT(!multiVolumeSwitch.isEmpty());
+
+    multiVolumeSwitch.replace(QLatin1String("$VolumeSize"), QString::number(volumeSize));
+
+    return multiVolumeSwitch;
 }
 
 QStringList CliInterface::copyFilesList(const QVariantList& files) const
@@ -764,10 +868,25 @@ QStringList CliInterface::copyFilesList(const QVariantList& files) const
     return filesList;
 }
 
-void CliInterface::failOperation()
+void CliInterface::killProcess(bool emitFinished)
 {
     // TODO: Would be good to unit test #304764/#304178.
-    doKill();
+
+    if (!m_process) {
+        return;
+    }
+
+    m_abortingOperation = !emitFinished;
+
+    // Give some time for the application to finish gracefully
+    if (!m_process->waitForFinished(5)) {
+        m_process->kill();
+
+        // It takes a few hundred ms for the process to be killed.
+        m_process->waitForFinished(1000);
+    }
+
+    m_abortingOperation = false;
 }
 
 bool CliInterface::passwordQuery()
@@ -780,7 +899,6 @@ bool CliInterface::passwordQuery()
         emit cancelled();
         // There is no process running, so finished() must be emitted manually.
         emit finished(false);
-        failOperation();
         return false;
     }
 
@@ -808,11 +926,6 @@ void CliInterface::readStdout(bool handleAll)
         return;
     }
 
-    //if the process is still not finished (m_process is appearantly not
-    //set to NULL if here), then the operation should definitely not be in
-    //the main thread as this would freeze everything. assert this.
-    Q_ASSERT(QThread::currentThread() != QApplication::instance()->thread());
-
     QByteArray dd = m_process->readAllStandardOutput();
     m_stdOutData += dd;
 
@@ -877,7 +990,7 @@ void CliInterface::handleLine(const QString& line)
     if ((m_operationMode == Copy || m_operationMode == Add) && m_param.contains(CaptureProgress) && m_param.value(CaptureProgress).toBool()) {
         //read the percentage
         int pos = line.indexOf(QLatin1Char( '%' ));
-        if (pos != -1 && pos > 1) {
+        if (pos > 1) {
             int percentage = line.midRef(pos - 2, 2).toInt();
             emit progress(float(percentage) / 100);
             return;
@@ -895,7 +1008,7 @@ void CliInterface::handleLine(const QString& line)
 
             if (query.responseCancelled()) {
                 emit cancelled();
-                failOperation();
+                killProcess();
                 return;
             }
 
@@ -910,7 +1023,7 @@ void CliInterface::handleLine(const QString& line)
         if (checkForErrorMessage(line, DiskFullPatterns)) {
             qCWarning(ARK) << "Found disk full message:" << line;
             emit error(i18nc("@info", "Extraction failed because the disk is full."));
-            failOperation();
+            killProcess();
             return;
         }
 
@@ -918,14 +1031,14 @@ void CliInterface::handleLine(const QString& line)
             qCWarning(ARK) << "Wrong password!";
             setPassword(QString());
             emit error(i18nc("@info", "Extraction failed: Incorrect password"));
-            failOperation();
+            killProcess();
             return;
         }
 
         if (checkForErrorMessage(line, ExtractionFailedPatterns)) {
             qCWarning(ARK) << "Error in extraction:" << line;
             emit error(i18n("Extraction failed because of an unexpected error."));
-            failOperation();
+            killProcess();
             return;
         }
 
@@ -944,7 +1057,7 @@ void CliInterface::handleLine(const QString& line)
 
             if (query.responseCancelled()) {
                 emit cancelled();
-                failOperation();
+                killProcess();
                 return;
             }
 
@@ -960,27 +1073,20 @@ void CliInterface::handleLine(const QString& line)
             qCWarning(ARK) << "Wrong password!";
             setPassword(QString());
             emit error(i18n("Incorrect password."));
-            failOperation();
+            killProcess();
             return;
         }
 
         if (checkForErrorMessage(line, ExtractionFailedPatterns)) {
             qCWarning(ARK) << "Error in extraction!!";
             emit error(i18n("Extraction failed because of an unexpected error."));
-            failOperation();
+            killProcess();
             return;
         }
 
         if (checkForErrorMessage(line, CorruptArchivePatterns)) {
             qCWarning(ARK) << "Archive corrupt";
             setCorrupt(true);
-            Kerfuffle::LoadCorruptQuery query(filename());
-            emit userQuery(&query);
-            query.waitForResponse();
-            if (!query.responseYes()) {
-                emit cancelled();
-                failOperation();
-            }
             return;
         }
 
@@ -991,6 +1097,23 @@ void CliInterface::handleLine(const QString& line)
         readListLine(line);
         return;
     }
+
+    if (m_operationMode == Test) {
+
+        if (checkForPasswordPromptMessage(line)) {
+            qCDebug(ARK) << "Found a password prompt";
+
+            emit error(i18n("Ark does not currently support testing this archive."));
+            killProcess();
+            return;
+        }
+
+        if (checkForTestSuccessMessage(line)) {
+            qCDebug(ARK) << "Test successful";
+            emit testSuccess();
+            return;
+        }
+    }
 }
 
 bool CliInterface::checkForPasswordPromptMessage(const QString& line)
@@ -1088,16 +1211,20 @@ bool CliInterface::checkForErrorMessage(const QString& line, int parameterIndex)
     return false;
 }
 
+bool CliInterface::checkForTestSuccessMessage(const QString& line)
+{
+    const QRegularExpression rx(m_param.value(TestPassedPattern).toString());
+    const QRegularExpressionMatch rxMatch = rx.match(line);
+    if (rxMatch.hasMatch()) {
+        return true;
+    }
+    return false;
+}
+
 bool CliInterface::doKill()
 {
     if (m_process) {
-        // Give some time for the application to finish gracefully
-        m_abortingOperation = true;
-        if (!m_process->waitForFinished(5)) {
-            m_process->kill();
-        }
-        m_abortingOperation = false;
-
+        killProcess(false);
         return true;
     }
 
@@ -1114,45 +1241,6 @@ bool CliInterface::doResume()
     return false;
 }
 
-void CliInterface::substituteListVariables(QStringList& params)
-{
-    for (int i = 0; i < params.size(); ++i) {
-        const QString parameter = params.at(i);
-
-        if (parameter == QLatin1String( "$Archive" )) {
-            params[i] = filename();
-        }
-
-        if (parameter == QLatin1String("$PasswordSwitch")) {
-            //if the PasswordSwitch argument has been added, we at least
-            //assume that the format of the switch has been added as well
-            Q_ASSERT(m_param.contains(PasswordSwitch));
-
-            //we will set it afterwards, if there is a password
-            params.removeAt(i);
-
-            QString pass = password();
-
-            if (!pass.isEmpty()) {
-                QStringList theSwitch = m_param.value(PasswordSwitch).toStringList();
-
-                for (int j = 0; j < theSwitch.size(); ++j) {
-                    //get the argument part
-                    QString newArg = theSwitch.at(j);
-
-                    //substitute the $Password
-                    newArg.replace(QLatin1String("$Password"), pass);
-
-                    //put it in the arg list
-                    params.insert(i + j, newArg);
-                    ++i;
-                }
-            }
-            --i;
-        }
-    }
-}
-
 QString CliInterface::escapeFileName(const QString& fileName) const
 {
     return fileName;
@@ -1172,6 +1260,47 @@ void CliInterface::writeToProcess(const QByteArray& data)
 #endif
 }
 
+bool CliInterface::addComment(const QString &comment)
+{
+    cacheParameterList();
+
+    m_operationMode = Comment;
+
+    m_commentTempFile = new QTemporaryFile;
+    if (!m_commentTempFile->open()) {
+        qCWarning(ARK) << "Failed to create temporary file for comment";
+        emit finished(false);
+        return false;
+    }
+
+    QTextStream stream(m_commentTempFile);
+    stream << comment << endl;
+    m_commentTempFile->close();
+
+    const auto args = substituteCommentVariables(m_param.value(CommentArgs).toStringList(),
+                                                 m_commentTempFile->fileName());
+
+    if (!runProcess(m_param.value(AddProgram).toStringList(), args)) {
+        return false;
+    }
+    m_comment = comment;
+    return true;
 }
 
+QString CliInterface::multiVolumeName() const
+{
+    QString oldSuffix = QMimeDatabase().suffixForFileName(filename());
+    QString name;
 
+    foreach (const QString &multiSuffix, m_param.value(MultiVolumeSuffix).toStringList()) {
+        QString newSuffix = multiSuffix;
+        newSuffix.replace(QStringLiteral("$Suffix"), oldSuffix);
+        name = filename().remove(oldSuffix).append(newSuffix);
+        if (QFileInfo(name).exists()) {
+            break;
+        }
+    }
+    return name;
+}
+
+}
