@@ -30,13 +30,16 @@
 
 #include "archiveinterface.h"
 #include "kerfuffle_export.h"
-#include <QtCore/QProcess>
+
+#include <QProcess>
 #include <QRegularExpression>
 
 class KProcess;
 class KPtyProcess;
 
 class QDir;
+class QTemporaryDir;
+class QTemporaryFile;
 
 namespace Kerfuffle
 {
@@ -97,9 +100,6 @@ enum CliInterfaceParameters {
      * $Archive - the path of the archive
      * $Files - the files selected to be extracted, if any
      * $PreservePathSwitch - the flag for extracting with full paths
-     * $RootNodeSwitch - the internal work dir in the archive (for example
-     * when the user has dragged a folder from the archive and wants it
-     * extracted relative to it)
      * $PasswordSwitch - the switch setting the password. Note that this
      * will not be inserted unless the listing function has emitted an
      * entry with the IsPasswordProtected property set to true.
@@ -127,14 +127,6 @@ enum CliInterfaceParameters {
      * stringlist ("--extractFull", "")
      */
     PreservePathSwitch,
-    /**
-     * QStringList (default empty)
-     * The format of the root node switch. The variable $Path will be
-     * substituted for the path string.
-     * Example: ("--internalPath=$Path)
-     * or ("--path", "$Path")
-     */
-    RootNodeSwitch,
     /**
      * QStringList (default empty)
      * The format of the root node switch. The variable $Password will be
@@ -146,6 +138,13 @@ enum CliInterfaceParameters {
      * or ("--password", "$Password")
      */
     PasswordSwitch,
+    /**
+     * QString
+     * The format of the compression level switch. The variable $CompressionLevel
+     * will be substituted for the level.
+     * Example: ("-mx=$CompressionLevel)
+     */
+    CompressionLevelSwitch,
     /**
      * QStringList
      * This is a stringlist with regexps, defining how to recognize the last
@@ -247,15 +246,26 @@ enum CliInterfaceParameters {
      */
     PasswordHeaderSwitch,
 
+    ///////////////[ COMMENT ]/////////////
+
     /**
-     * QStringList (default empty)
-     * Encrypt the archive header without providing a password string.
-     * It uses $Password from either PasswordSwitch or PasswordHeaderSwitch.
-     * However, there is the $Enabled variable
-     * which is substituted with either 'on' or 'off'.
-     * Example (7z plugin): ("-mhe=$Enabled")
+     * QStringList
+     * The arguments that are passed to AddProgram when adding
+     * a comment.
      */
-    EncryptHeaderSwitch
+    CommentArgs,
+    /**
+     * QString
+     * The variable $CommentFile will be substituted for the file
+     * containing the comment.
+     * Example (rar plugin): -z$CommentFile
+     */
+    CommentSwitch,
+    TestProgram,
+    TestArgs,
+    TestPassedPattern,
+    MultiVolumeSwitch,
+    MultiVolumeSuffix
 };
 
 typedef QHash<int, QVariant> ParameterList;
@@ -266,7 +276,7 @@ class KERFUFFLE_EXPORT CliInterface : public ReadWriteArchiveInterface
 
 public:
     enum OperationMode  {
-        List, Copy, Add, Delete
+        List, Copy, Add, Delete, Comment, Test
     };
     OperationMode m_operationMode;
 
@@ -277,6 +287,8 @@ public:
     virtual bool copyFiles(const QList<QVariant>& files, const QString& destinationDirectory, const ExtractionOptions& options) Q_DECL_OVERRIDE;
     virtual bool addFiles(const QStringList & files, const CompressionOptions& options) Q_DECL_OVERRIDE;
     virtual bool deleteFiles(const QList<QVariant> & files) Q_DECL_OVERRIDE;
+    virtual bool addComment(const QString &comment) Q_DECL_OVERRIDE;
+    virtual bool testArchive() Q_DECL_OVERRIDE;
 
     virtual void resetParsing() = 0;
     virtual ParameterList parameterList() const = 0;
@@ -284,25 +296,6 @@ public:
     bool doKill() Q_DECL_OVERRIDE;
     bool doSuspend() Q_DECL_OVERRIDE;
     bool doResume() Q_DECL_OVERRIDE;
-    bool findExecutables(bool isReadWrite) Q_DECL_OVERRIDE;
-    bool isCliBased() const Q_DECL_OVERRIDE;
-
-    /**
-     * Returns the list of characters which are preceded by a
-     * backslash when a file name in an archive is passed to
-     * a program.
-     *
-     * @see setEscapedCharacters().
-     */
-    QString escapedCharacters(); // FIXME not implemented?
-
-    /**
-     * Sets which characters will be preceded by a backslash when
-     * a file name in an archive is passed to a program.
-     *
-     * @see escapedCharacters().
-     */
-    void setEscapedCharacters(const QString& characters); // FIXME not implemented?
 
     /**
      * Sets if the listing should include empty lines.
@@ -313,53 +306,66 @@ public:
 
     /**
      * Move all files from @p tmpDir to @p destDir, preserving paths if @p preservePaths is true.
-     * @return Whether the operation has been successfull.
+     * @return Whether the operation has been successful.
      */
     bool moveToDestination(const QDir &tempDir, const QDir &destDir, bool preservePaths);
 
-    QStringList substituteCopyVariables(const QStringList &extractArgs, const QVariantList &files, bool preservePaths, const QString &password, const QString &rootNode);
-
+    QStringList substituteListVariables(const QStringList &listArgs, const QString &password);
+    QStringList substituteCopyVariables(const QStringList &extractArgs, const QVariantList &files, bool preservePaths, const QString &password);
+    QStringList substituteAddVariables(const QStringList &addArgs, const QStringList &files, const QString &password, bool encryptHeader, int compLevel, ulong volumeSize);
     QStringList substituteDeleteVariables(const QStringList &deleteArgs, const QVariantList &files, const QString &password);
+    QStringList substituteCommentVariables(const QStringList &commentArgs, const QString &commentFile);
+    QStringList substituteTestVariables(const QStringList &testArgs, const QString &password);
 
     /**
      * @return The preserve path switch, according to the @p preservePaths extraction option.
      */
     QString preservePathSwitch(bool preservePaths) const;
 
+    /**
+     * @return The password header-switch with the given @p password.
+     */
+    virtual QStringList passwordHeaderSwitch(const QString& password) const;
+
     /**
      * @return The password switch with the given @p password.
      */
     QStringList passwordSwitch(const QString& password) const;
 
     /**
-     * @return The root node switch with the given @p rootNode.
+     * @return The compression level switch with the given @p level.
      */
-    QStringList rootNodeSwitch(const QString& rootNode) const;
+    QString compressionLevelSwitch(int level) const;
+
+    QString multiVolumeSwitch(ulong volumeSize) const;
 
     /**
      * @return The list of selected files to extract.
      */
     QStringList copyFilesList(const QVariantList& files) const;
 
+    QString multiVolumeName() const;
+
 protected:
 
     virtual void handleLine(const QString& line);
     virtual void cacheParameterList();
-    void substituteListVariables(QStringList& params);
 
     /**
      * Run @p programName with the given @p arguments.
-     * The method waits until @p programName is finished to exit.
      *
      * @param programName The program that will be run (not the whole path).
      * @param arguments A list of arguments that will be passed to the program.
      *
-     * @return @c true if the program was found and the process ran correctly,
-     *         @c false otherwise.
+     * @return @c true if the program was found and the process was started correctly,
+     *         @c false otherwise (in which case finished(false) is emitted).
      */
     bool runProcess(const QStringList& programNames, const QStringList& arguments);
 
-    void failOperation();
+    /**
+     * Kill the running process. The finished signal is emitted according to @p emitFinished.
+     */
+    void killProcess(bool emitFinished = true);
 
     /**
      * Ask the password *before* running any process.
@@ -391,6 +397,7 @@ private:
 
     bool handleFileExistsMessage(const QString& filename);
     bool checkForErrorMessage(const QString& line, int parameterIndex);
+    bool checkForTestSuccessMessage(const QString& line);
 
     /**
      * Performs any additional escaping and processing on @p fileName
@@ -415,6 +422,8 @@ private:
      */
     bool isEmptyDir(const QDir &dir);
 
+    void copyProcessCleanup();
+
     QByteArray m_stdOutData;
     QRegularExpression m_passwordPromptPattern;
     QHash<int, QList<QRegularExpression> > m_patternCache;
@@ -430,8 +439,17 @@ private:
     bool m_abortingOperation;
     QString m_storedFileName;
 
+    CompressionOptions m_compressionOptions;
+    QString m_oldWorkingDir;
+    QString m_extractDestDir;
+    QTemporaryDir *m_extractTempDir;
+    QTemporaryFile *m_commentTempFile;
+    QVariantList m_copiedFiles;
+
 private slots:
     void processFinished(int exitCode, QProcess::ExitStatus exitStatus);
+    void copyProcessFinished(int exitCode, QProcess::ExitStatus exitStatus);
+
 };
 }
 
