@@ -26,20 +26,18 @@
 #include "kerfuffle/archive_kerfuffle.h"
 #include "kerfuffle/jobs.h"
 
-#include <QDebug>
 #include <KIconLoader>
 #include <KLocalizedString>
 #include <kio/global.h>
 
-#include <QDir>
-#include <QFont>
-#include <QLatin1String>
-#include <QList>
+#include <QDateTime>
+#include <QDBusConnection>
+#include <QElapsedTimer>
 #include <QMimeData>
+#include <QMimeDatabase>
 #include <QPersistentModelIndex>
-#include <QPixmap>
-#include <QIcon>
-#include <QtDBus/QtDBus>
+#include <QRegularExpression>
+#include <QUrl>
 
 using namespace Kerfuffle;
 
@@ -286,6 +284,8 @@ ArchiveModel::ArchiveModel(const QString &dbusPathName, QObject *parent)
     : QAbstractItemModel(parent)
     , m_rootNode(new ArchiveDirNode(0, ArchiveEntry()))
     , m_dbusPathName(dbusPathName)
+    , m_numberOfFiles(0)
+    , m_numberOfFolders(0)
 {
 }
 
@@ -499,10 +499,8 @@ int ArchiveModel::rowCount(const QModelIndex &parent) const
 
 int ArchiveModel::columnCount(const QModelIndex &parent) const
 {
+    Q_UNUSED(parent)
     return m_showColumns.size();
-    if (parent.isValid()) {
-        return static_cast<ArchiveNode*>(parent.internalPointer())->entry().size();
-    }
 }
 
 void ArchiveModel::sort(int column, Qt::SortOrder order)
@@ -708,8 +706,6 @@ QModelIndex ArchiveModel::indexForNode(ArchiveNode *node)
 
 void ArchiveModel::slotEntryRemoved(const QString & path)
 {
-    qCDebug(ARK) << "Removed node at path " << path;
-
     const QString entryFileName(cleanFileName(path));
     if (entryFileName.isEmpty()) {
         return;
@@ -728,8 +724,6 @@ void ArchiveModel::slotEntryRemoved(const QString & path)
         parent->removeEntryAt(entry->row());
 
         endRemoveRows();
-    } else {
-        qCDebug(ARK) << "Did not find the removed node";
     }
 }
 
@@ -921,6 +915,30 @@ ExtractJob* ArchiveModel::extractFiles(const QList<QVariant>& files, const QStri
     return newJob;
 }
 
+Kerfuffle::PreviewJob *ArchiveModel::preview(const QString& file) const
+{
+    Q_ASSERT(m_archive);
+    PreviewJob *job = m_archive->preview(file);
+    connect(job, &Job::userQuery, this, &ArchiveModel::slotUserQuery);
+    return job;
+}
+
+OpenJob *ArchiveModel::open(const QString& file) const
+{
+    Q_ASSERT(m_archive);
+    OpenJob *job = m_archive->open(file);
+    connect(job, &Job::userQuery, this, &ArchiveModel::slotUserQuery);
+    return job;
+}
+
+OpenWithJob *ArchiveModel::openWith(const QString& file) const
+{
+    Q_ASSERT(m_archive);
+    OpenWithJob *job = m_archive->openWith(file);
+    connect(job, &Job::userQuery, this, &ArchiveModel::slotUserQuery);
+    return job;
+}
+
 AddJob* ArchiveModel::addFiles(const QStringList & filenames, const CompressionOptions& options)
 {
     if (!m_archive) {
@@ -976,7 +994,6 @@ void ArchiveModel::slotCleanupEmptyDirs()
     while (!queue.isEmpty()) {
         QPersistentModelIndex node = queue.takeFirst();
         ArchiveEntry entry = entryForIndex(node);
-        qCDebug(ARK) << "Trying " << entry[FileName].toString();
 
         if (!hasChildren(node)) {
             if (!entry.contains(InternalID)) {
@@ -998,4 +1015,51 @@ void ArchiveModel::slotCleanupEmptyDirs()
     }
 }
 
+void ArchiveModel::countEntriesAndSize() {
 
+    // This function is used to count the number of folders/files and
+    // the total compressed size. This is needed for PropertiesDialog
+    // to update the corresponding values after adding/deleting files.
+
+    // When ArchiveModel has been properly fixed, this code can likely
+    // be removed.
+
+    m_numberOfFiles = 0;
+    m_numberOfFolders = 0;
+    m_uncompressedSize = 0;
+
+    QElapsedTimer timer;
+    timer.start();
+
+    traverseAndCountDirNode(m_rootNode);
+
+    qCDebug(ARK) << "Time to count entries and size:" << timer.elapsed() << "ms";
+}
+
+void ArchiveModel::traverseAndCountDirNode(ArchiveDirNode *dir)
+{
+    foreach(ArchiveNode *node, dir->entries()) {
+        if (node->isDir()) {
+            traverseAndCountDirNode(dynamic_cast<ArchiveDirNode*>(node));
+            m_numberOfFolders++;
+        } else {
+            m_numberOfFiles++;
+            m_uncompressedSize += node->entry()[Size].toULongLong();
+        }
+    }
+}
+
+qulonglong ArchiveModel::numberOfFiles() const
+{
+    return m_numberOfFiles;
+}
+
+qulonglong ArchiveModel::numberOfFolders() const
+{
+    return m_numberOfFolders;
+}
+
+qulonglong ArchiveModel::uncompressedSize() const
+{
+    return m_uncompressedSize;
+}
