@@ -72,6 +72,10 @@ public:
     Kerfuffle::ExtractJob* extractFile(const QVariant& fileName, const QString& destinationDir, const Kerfuffle::ExtractionOptions& options = Kerfuffle::ExtractionOptions()) const;
     Kerfuffle::ExtractJob* extractFiles(const QList<QVariant>& files, const QString& destinationDir, const Kerfuffle::ExtractionOptions& options = Kerfuffle::ExtractionOptions()) const;
 
+    Kerfuffle::PreviewJob* preview(const QString& file) const;
+    Kerfuffle::OpenJob* open(const QString& file) const;
+    Kerfuffle::OpenWithJob* openWith(const QString& file) const;
+
     Kerfuffle::AddJob* addFiles(const QStringList & paths, const Kerfuffle::CompressionOptions& options = Kerfuffle::CompressionOptions());
     Kerfuffle::DeleteJob* deleteFiles(const QList<QVariant> & files);
 
@@ -81,6 +85,11 @@ public:
      */
     void encryptArchive(const QString &password, bool encryptHeader);
 
+    void countEntriesAndSize();
+    qulonglong numberOfFiles() const;
+    qulonglong numberOfFolders() const;
+    qulonglong uncompressedSize() const;
+
 signals:
     void loadingStarted();
     void loadingFinished(KJob *);
@@ -120,12 +129,18 @@ private:
     void insertNode(ArchiveNode *node, InsertBehaviour behaviour = NotifyViews);
     void newEntry(const Kerfuffle::ArchiveEntry& entry, InsertBehaviour behaviour);
 
+    void traverseAndCountDirNode(ArchiveDirNode *dir);
+
     QList<Kerfuffle::ArchiveEntry> m_newArchiveEntries; // holds entries from opening a new archive until it's totally open
     QList<int> m_showColumns;
     QScopedPointer<Kerfuffle::Archive> m_archive;
     ArchiveDirNode *m_rootNode;
 
     QString m_dbusPathName;
+
+    qulonglong m_numberOfFiles;
+    qulonglong m_numberOfFolders;
+    qulonglong m_uncompressedSize;
 };
 
 #endif // ARCHIVEMODEL_H
