@@ -23,6 +23,8 @@
 
 #include "part.h"
 #include "ark_debug.h"
+#include "adddialog.h"
+#include "archiveformat.h"
 #include "archivemodel.h"
 #include "archiveview.h"
 #include "arkviewer.h"
@@ -36,6 +38,7 @@
 #include "kerfuffle/settings.h"
 #include "kerfuffle/previewsettingspage.h"
 #include "kerfuffle/propertiesdialog.h"
+#include "pluginmanager.h"
 
 #include <KAboutData>
 #include <KActionCollection>
@@ -64,7 +67,6 @@
 #include <QPointer>
 #include <QSplitter>
 #include <QTimer>
-#include <QtDBus/QtDBus>
 #include <QFileDialog>
 #include <QIcon>
 #include <QInputDialog>
@@ -74,7 +76,7 @@
 
 using namespace Kerfuffle;
 
-K_PLUGIN_FACTORY(Factory, registerPlugin<Ark::Part>();)
+K_PLUGIN_FACTORY_WITH_JSON(Factory, "ark_part.json", registerPlugin<Ark::Part>();)
 
 namespace Ark
 {
@@ -115,11 +117,29 @@ Part::Part(QWidget *parentWidget, QObject *parent, const QVariantList& args)
     vbox->addWidget(m_commentView);
     m_commentBox->setLayout(vbox);
 
+    m_messageWidget = new KMessageWidget(parentWidget);
+    m_messageWidget->hide();
+
+    m_commentMsgWidget = new KMessageWidget();
+    m_commentMsgWidget->setText(i18n("Comment has been modified."));
+    m_commentMsgWidget->setMessageType(KMessageWidget::Information);
+    m_commentMsgWidget->setCloseButtonVisible(false);
+    m_commentMsgWidget->hide();
+
+    QAction *saveAction = new QAction(i18n("Save"), m_commentMsgWidget);
+    m_commentMsgWidget->addAction(saveAction);
+    connect(saveAction, &QAction::triggered, this, &Part::slotAddComment);
+
+    m_commentBox->layout()->addWidget(m_commentMsgWidget);
+
+    connect(m_commentView, &QPlainTextEdit::textChanged, this, &Part::slotCommentChanged);
+
     setWidget(mainWidget);
     mainWidget->setLayout(m_vlayout);
 
     // Configure the QVBoxLayout and add widgets
     m_vlayout->setContentsMargins(0,0,0,0);
+    m_vlayout->addWidget(m_messageWidget);
     m_vlayout->addWidget(m_splitter);
 
     // Vertical QSplitter for the file view and comment field.
@@ -147,8 +167,8 @@ Part::Part(QWidget *parentWidget, QObject *parent, const QVariantList& args)
             this, &Part::slotLoadingStarted);
     connect(m_model, &ArchiveModel::loadingFinished,
             this, &Part::slotLoadingFinished);
-    connect(m_model, SIGNAL(droppedFiles(QStringList,QString)),
-            this, SLOT(slotAddFiles(QStringList,QString)));
+    connect(m_model, &ArchiveModel::droppedFiles,
+            this, static_cast<void (Part::*)(const QStringList&, const QString&)>(&Part::slotAddFiles));
     connect(m_model, &ArchiveModel::error,
             this, &Part::slotError);
 
@@ -156,8 +176,8 @@ Part::Part(QWidget *parentWidget, QObject *parent, const QVariantList& args)
             this, &Part::setBusyGui);
     connect(this, &Part::ready,
             this, &Part::setReadyGui);
-    connect(this, SIGNAL(completed()),
-            this, SLOT(setFileNameFromArchive()));
+    connect(this, static_cast<void (KParts::ReadOnlyPart::*)()>(&KParts::ReadOnlyPart::completed),
+            this, &Part::setFileNameFromArchive);
 
     m_statusBarExtension = new KParts::StatusBarExtension(this);
 
@@ -180,6 +200,19 @@ Part::~Part()
     m_extractAction->menu()->deleteLater();
 }
 
+void Part::slotCommentChanged()
+{
+    if (!m_model->archive()) {
+        return;
+    }
+
+    if (m_commentMsgWidget->isHidden() && m_commentView->toPlainText() != m_model->archive()->comment()) {
+        m_commentMsgWidget->animatedShow();
+    } else if (m_commentMsgWidget->isVisible() && m_commentView->toPlainText() == m_model->archive()->comment()) {
+        m_commentMsgWidget->hide();
+    }
+}
+
 KAboutData *Part::createAboutData()
 {
     return new KAboutData(QStringLiteral("ark"),
@@ -284,7 +317,7 @@ void Part::setupActions()
     // pass the OpenFileMode as argument to the slot.
     m_signalMapper = new QSignalMapper;
 
-    m_showInfoPanelAction = new KToggleAction(i18nc("@action:inmenu", "Show information panel"), this);
+    m_showInfoPanelAction = new KToggleAction(i18nc("@action:inmenu", "Show Information Panel"), this);
     actionCollection()->addAction(QStringLiteral( "show-infopanel" ), m_showInfoPanelAction);
     m_showInfoPanelAction->setChecked(ArkSettings::showInfoPanel());
     connect(m_showInfoPanelAction, &QAction::triggered,
@@ -332,18 +365,11 @@ void Part::setupActions()
 
     m_addFilesAction = actionCollection()->addAction(QStringLiteral("add"));
     m_addFilesAction->setIcon(QIcon::fromTheme(QStringLiteral("archive-insert")));
-    m_addFilesAction->setText(i18n("Add &File..."));
+    m_addFilesAction->setText(i18n("Add &Files..."));
     m_addFilesAction->setToolTip(i18nc("@info:tooltip", "Click to add files to the archive"));
     connect(m_addFilesAction, SIGNAL(triggered(bool)),
             this, SLOT(slotAddFiles()));
 
-    m_addDirAction = actionCollection()->addAction(QStringLiteral("add-dir"));
-    m_addDirAction->setIcon(QIcon::fromTheme(QStringLiteral("archive-insert-directory")));
-    m_addDirAction->setText(i18n("Add Fo&lder..."));
-    m_addDirAction->setToolTip(i18nc("@info:tooltip", "Click to add a folder to the archive"));
-    connect(m_addDirAction, &QAction::triggered,
-            this, &Part::slotAddDir);
-
     m_deleteFilesAction = actionCollection()->addAction(QStringLiteral("delete"));
     m_deleteFilesAction->setIcon(QIcon::fromTheme(QStringLiteral("archive-remove")));
     m_deleteFilesAction->setText(i18n("De&lete"));
@@ -360,7 +386,21 @@ void Part::setupActions()
     connect(m_propertiesAction, &QAction::triggered,
             this, &Part::slotShowProperties);
 
-    connect(m_signalMapper, SIGNAL(mapped(int)), this, SLOT(slotOpenEntry(int)));
+    m_editCommentAction = actionCollection()->addAction(QStringLiteral("edit_comment"));
+    m_editCommentAction->setIcon(QIcon::fromTheme(QStringLiteral("document-edit")));
+    actionCollection()->setDefaultShortcut(m_editCommentAction, Qt::ALT + Qt::Key_C);
+    m_editCommentAction->setToolTip(i18nc("@info:tooltip", "Click to add or edit comment"));
+    connect(m_editCommentAction, &QAction::triggered, this, &Part::slotShowComment);
+
+    m_testArchiveAction = actionCollection()->addAction(QStringLiteral("test_archive"));
+    m_testArchiveAction->setIcon(QIcon::fromTheme(QStringLiteral("checkmark")));
+    m_testArchiveAction->setText(i18nc("@action:inmenu", "&Test Integrity"));
+    actionCollection()->setDefaultShortcut(m_testArchiveAction, Qt::ALT + Qt::Key_T);
+    m_testArchiveAction->setToolTip(i18nc("@info:tooltip", "Click to test the archive for integrity"));
+    connect(m_testArchiveAction, &QAction::triggered, this, &Part::slotTestArchive);
+
+    connect(m_signalMapper, static_cast<void (QSignalMapper::*)(int)>(&QSignalMapper::mapped),
+            this, &Part::slotOpenEntry);
 
     updateActions();
     updateQuickExtractMenu(m_extractArchiveAction);
@@ -369,10 +409,30 @@ void Part::setupActions()
 
 void Part::updateActions()
 {
-    bool isWritable = m_model->archive() && !m_model->archive()->isReadOnly();
+    bool isWritable = m_model->archive() && !m_model->archive()->isReadOnly() &&
+                      !(m_model->rowCount() > 0 && m_model->archive()->isMultiVolume());
     bool isDirectory = m_model->entryForIndex(m_view->selectionModel()->currentIndex())[IsDirectory].toBool();
     int selectedEntriesCount = m_view->selectionModel()->selectedRows().count();
 
+    // We disable adding files if the archive is encrypted but the password is
+    // unknown (this happens when opening existing non-he password-protected
+    // archives). If we added files they would not get encrypted resulting in an
+    // archive with a mixture of encrypted and unencrypted files.
+    const bool isEncryptedButUnknownPassword = m_model->archive() &&
+                                               m_model->archive()->encryptionType() != Archive::Unencrypted &&
+                                               m_model->archive()->password().isEmpty();
+
+    if (isEncryptedButUnknownPassword) {
+        m_addFilesAction->setToolTip(xi18nc("@info:tooltip",
+                                            "Adding files to existing password-protected archives with no header-encryption is currently not supported."
+                                            "<nl/><nl/>Extract the files and create a new archive if you want to add files."));
+        m_testArchiveAction->setToolTip(xi18nc("@info:tooltip",
+                                               "Testing password-protected archives with no header-encryption is currently not supported."));
+    } else {
+        m_addFilesAction->setToolTip(i18nc("@info:tooltip", "Click to add files to the archive"));
+        m_testArchiveAction->setToolTip(i18nc("@info:tooltip", "Click to test the archive for integrity"));
+    }
+
     // Figure out if entry size is larger than preview size limit.
     const int maxPreviewSize = ArkSettings::previewFileSizeLimit() * 1024 * 1024;
     const bool limit = ArkSettings::limitPreviewFileSize();
@@ -390,9 +450,8 @@ void Part::updateActions()
     m_saveAsAction->setEnabled(!isBusy() &&
                                m_model->rowCount() > 0);
     m_addFilesAction->setEnabled(!isBusy() &&
-                                 isWritable);
-    m_addDirAction->setEnabled(!isBusy() &&
-                               isWritable);
+                                 isWritable &&
+                                 !isEncryptedButUnknownPassword);
     m_deleteFilesAction->setEnabled(!isBusy() &&
                                     isWritable &&
                                     (selectedEntriesCount > 0));
@@ -406,6 +465,82 @@ void Part::updateActions()
                                      (selectedEntriesCount == 1));
     m_propertiesAction->setEnabled(!isBusy() &&
                                    m_model->archive());
+
+    m_commentView->setEnabled(!isBusy());
+    m_commentMsgWidget->setEnabled(!isBusy());
+
+    m_editCommentAction->setEnabled(false);
+    m_testArchiveAction->setEnabled(false);
+
+    if (m_model->archive()) {
+        const KPluginMetaData metadata = PluginManager().preferredPluginFor(m_model->archive()->mimeType())->metaData();
+        bool supportsWriteComment = ArchiveFormat::fromMetadata(m_model->archive()->mimeType(), metadata).supportsWriteComment();
+        m_editCommentAction->setEnabled(!isBusy() &&
+                                        supportsWriteComment);
+        m_commentView->setReadOnly(!supportsWriteComment);
+        m_editCommentAction->setText(m_model->archive()->hasComment() ? i18nc("@action:inmenu mutually exclusive with Add &Comment", "Edit &Comment") :
+                                                                        i18nc("@action:inmenu mutually exclusive with Edit &Comment", "Add &Comment"));
+
+        bool supportsTesting = ArchiveFormat::fromMetadata(m_model->archive()->mimeType(), metadata).supportsTesting();
+        m_testArchiveAction->setEnabled(!isBusy() &&
+                                        supportsTesting &&
+                                        !isEncryptedButUnknownPassword);
+    } else {
+        m_commentView->setReadOnly(true);
+        m_editCommentAction->setText(i18nc("@action:inmenu mutually exclusive with Edit &Comment", "Add &Comment"));
+    }
+}
+
+void Part::slotShowComment()
+{
+    if (!m_commentBox->isVisible()) {
+        m_commentBox->show();
+        m_commentSplitter->setSizes(QList<int>() << m_view->height() * 0.6 << 1);
+    }
+    m_commentView->setFocus();
+}
+
+void Part::slotAddComment()
+{
+    CommentJob *job = m_model->archive()->addComment(m_commentView->toPlainText());
+    if (!job) {
+        return;
+    }
+    registerJob(job);
+    job->start();
+    m_commentMsgWidget->hide();
+    if (m_commentView->toPlainText().isEmpty()) {
+        m_commentBox->hide();
+    }
+}
+
+void Part::slotTestArchive()
+{
+    TestJob *job = m_model->archive()->testArchive();
+    if (!job) {
+        return;
+    }
+    registerJob(job);
+    connect(job, &KJob::result, this, &Part::slotTestingDone);
+    job->start();
+}
+
+void Part::resetGui()
+{
+    m_messageWidget->hide();
+    m_commentView->clear();
+    m_commentBox->hide();
+}
+
+void Part::slotTestingDone(KJob* job)
+{
+    if (job->error() && job->error() != KJob::KilledJobError) {
+        KMessageBox::error(widget(), job->errorString());
+    } else if (static_cast<TestJob*>(job)->testSucceeded()) {
+        KMessageBox::information(widget(), i18n("The archive passed the integrity test."), i18n("Test Results"));
+    } else {
+        KMessageBox::error(widget(), i18n("The archive failed the integrity test."), i18n("Test Results"));
+    }
 }
 
 void Part::updateQuickExtractMenu(QAction *extractAction)
@@ -503,11 +638,14 @@ bool Part::openFile()
 {
     qCDebug(ARK) << "Attempting to open archive" << localFilePath();
 
+    resetGui();
+
     if (!isLocalFileValid()) {
         return false;
     }
 
-    QScopedPointer<Kerfuffle::Archive> archive(Kerfuffle::Archive::create(localFilePath(), m_model));
+    const QString fixedMimeType = arguments().metaData()[QStringLiteral("fixedMimeType")];
+    QScopedPointer<Kerfuffle::Archive> archive(Kerfuffle::Archive::create(localFilePath(), fixedMimeType, m_model));
     Q_ASSERT(archive);
 
     if (archive->error() == NoPlugin) {
@@ -526,6 +664,10 @@ bool Part::openFile()
 
     Q_ASSERT(archive->isValid());
 
+    if (arguments().metaData().contains(QStringLiteral("volumeSize"))) {
+        archive.data()->setMultiVolume(true);
+    }
+
     // Plugin loaded successfully.
     KJob *job = m_model->setArchive(archive.take());
     if (job) {
@@ -676,8 +818,7 @@ void Part::slotLoadingFinished(KJob *job)
 
     if (!m_model->archive()->comment().isEmpty()) {
         m_commentView->setPlainText(m_model->archive()->comment());
-        m_commentBox->show();
-        m_commentSplitter->setSizes(QList<int>() << m_view->height() * 0.6 << 1);
+        slotShowComment();
     } else {
         m_commentView->clear();
         m_commentBox->hide();
@@ -751,37 +892,36 @@ void Part::slotOpenEntry(int mode)
 
     // Extract the entry.
     if (!entry.isEmpty()) {
-        Kerfuffle::ExtractionOptions options;
-        options[QStringLiteral("PreservePaths")] = true;
 
-        m_tmpOpenDirList.append(new QTemporaryDir);
         m_openFileMode = static_cast<OpenFileMode>(mode);
-        ExtractJob *job = m_model->extractFile(entry[InternalID], m_tmpOpenDirList.last()->path(), options);
+        KJob *job = Q_NULLPTR;
+
+        if (m_openFileMode == Preview) {
+            job = m_model->preview(entry[InternalID].toString());
+            connect(job, &KJob::result, this, &Part::slotPreviewExtractedEntry);
+        } else {
+            const QString file = entry[InternalID].toString();
+            job = (m_openFileMode == OpenFile) ? m_model->open(file) : m_model->openWith(file);
+            connect(job, &KJob::result, this, &Part::slotOpenExtractedEntry);
+        }
 
         registerJob(job);
-        connect(job, &KJob::result,
-                this, &Part::slotOpenExtractedEntry);
         job->start();
     }
 }
 
 void Part::slotOpenExtractedEntry(KJob *job)
 {
-    // FIXME: the error checking here isn't really working
-    //        if there's an error or an overwrite dialog,
-    //        the preview dialog will be launched anyway
     if (!job->error()) {
-        const ArchiveEntry& entry =
-            m_model->entryForIndex(m_view->selectionModel()->currentIndex());
 
-        ExtractJob *extractJob = qobject_cast<ExtractJob*>(job);
-        Q_ASSERT(extractJob);
-        QString fullName = extractJob->destinationDirectory() + QLatin1Char('/') + entry[FileName].toString();
+        OpenJob *openJob = qobject_cast<OpenJob*>(job);
+        Q_ASSERT(openJob);
 
-        // Make sure a maliciously crafted archive with parent folders named ".." do
-        // not cause the previewed file path to be located outside the temporary
-        // directory, resulting in a directory traversal issue.
-        fullName.remove(QStringLiteral("../"));
+        // Since the user could modify the file (unlike the Preview case),
+        // we'll need to manually delete the temp dir in the Part destructor.
+        m_tmpOpenDirList << openJob->tempDir();
+
+        const QString fullName = openJob->validatedFilePath();
 
         bool isWritable = m_model->archive() && !m_model->archive()->isReadOnly();
 
@@ -791,40 +931,34 @@ void Part::slotOpenExtractedEntry(KJob *job)
             QFile::setPermissions(fullName, QFileDevice::ReadOwner | QFileDevice::ReadGroup | QFileDevice::ReadOther);
         }
 
-        // TODO: get rid of m_openFileMode by extending ExtractJob with a
-        // Preview/OpenJob. This would prevent race conditions if we ever stop
-        // disabling the whole UI while extracting a file to preview it.
-        if (m_openFileMode != Preview && isWritable) {
+        if (isWritable) {
             m_fileWatcher = new QFileSystemWatcher;
             connect(m_fileWatcher, &QFileSystemWatcher::fileChanged, this, &Part::slotWatchedFileModified);
-        }
-
-        QMimeDatabase db;
-        switch (m_openFileMode) {
-
-        case Preview:
-            ArkViewer::view(fullName);
-            break;
-        case OpenFile:
-            KRun::runUrl(QUrl::fromUserInput(fullName,
-                                             QString(),
-                                             QUrl::AssumeLocalFile),
-                         db.mimeTypeForFile(fullName).name(),
-                         widget());
-            break;
-        case OpenFileWith:
-            QList<QUrl> list;
-            list.append(QUrl::fromUserInput(fullName,
-                                            QString(),
-                                            QUrl::AssumeLocalFile));
-            KRun::displayOpenWithDialog(list, widget(), false);
-
-            break;
-        }
-        if (m_openFileMode != Preview && isWritable) {
             m_fileWatcher->addPath(fullName);
         }
 
+        if (qobject_cast<OpenWithJob*>(job)) {
+            const QList<QUrl> urls = {QUrl::fromUserInput(fullName, QString(), QUrl::AssumeLocalFile)};
+            KRun::displayOpenWithDialog(urls, widget());
+        } else {
+            KRun::runUrl(QUrl::fromUserInput(fullName, QString(), QUrl::AssumeLocalFile),
+                         QMimeDatabase().mimeTypeForFile(fullName).name(),
+                         widget());
+        }
+    } else if (job->error() != KJob::KilledJobError) {
+        KMessageBox::error(widget(), job->errorString());
+    }
+    setReadyGui();
+}
+
+void Part::slotPreviewExtractedEntry(KJob *job)
+{
+    if (!job->error()) {
+        PreviewJob *previewJob = qobject_cast<PreviewJob*>(job);
+        Q_ASSERT(previewJob);
+
+        ArkViewer::view(previewJob->validatedFilePath());
+
     } else if (job->error() != KJob::KilledJobError) {
         KMessageBox::error(widget(), job->errorString());
     }
@@ -946,7 +1080,7 @@ void Part::slotShowExtractionDialog()
         }
         options[QStringLiteral("FollowExtractionDialogSettings")] = true;
 
-        const QString destinationDirectory = dialog.data()->destinationDirectory().toDisplayString(QUrl::PreferLocalFile);
+        const QString destinationDirectory = dialog.data()->destinationDirectory().toLocalFile();
         ExtractJob *job = m_model->extractFiles(files, destinationDirectory, options);
         registerJob(job);
 
@@ -1067,24 +1201,13 @@ void Part::slotAddFiles(const QStringList& filesToAdd, const QString& path)
 
     qCDebug(ARK) << "Adding " << filesToAdd << " to " << path;
 
-    // Add a trailing slash to directories.
-    QStringList cleanFilesToAdd(filesToAdd);
-    for (int i = 0; i < cleanFilesToAdd.size(); ++i) {
-        QString& file = cleanFilesToAdd[i];
-        if (QFileInfo(file).isDir()) {
-            if (!file.endsWith(QLatin1Char( '/' ))) {
-                file += QLatin1Char( '/' );
-            }
-        }
-    }
-
     // GlobalWorkDir is used by AddJob and should contain the part of the
     // absolute path of files to be added that should NOT be included in the
     // directory structure within the archive.
     // Example: We add file "/home/user/somedir/somefile.txt" and want the file
     // to have the relative path within the archive "somedir/somefile.txt".
     // GlobalWorkDir is then: "/home/user"
-    QString globalWorkDir = cleanFilesToAdd.first();
+    QString globalWorkDir = filesToAdd.first();
 
     // path represents the path of the file within the archive. This needs to
     // be removed from globalWorkDir, otherwise the files will be added to the
@@ -1098,14 +1221,15 @@ void Part::slotAddFiles(const QStringList& filesToAdd, const QString& path)
         globalWorkDir.chop(1);
     }
 
+    CompressionOptions options(m_model->archive()->compressionOptions());
+
     // Now take the absolute path of the parent directory.
     globalWorkDir = QFileInfo(globalWorkDir).dir().absolutePath();
 
     qCDebug(ARK) << "Detected GlobalWorkDir to be " << globalWorkDir;
-    CompressionOptions options;
     options[QStringLiteral("GlobalWorkDir")] = globalWorkDir;
 
-    AddJob *job = m_model->addFiles(cleanFilesToAdd, options);
+    AddJob *job = m_model->addFiles(filesToAdd, options);
     if (!job) {
         return;
     }
@@ -1118,6 +1242,22 @@ void Part::slotAddFiles(const QStringList& filesToAdd, const QString& path)
 
 void Part::slotAddFiles()
 {
+    // If compression options are already set, we dont use the values from CreateDialog.
+    CompressionOptions opts;
+    if (m_model->archive()->compressionOptions().isEmpty()) {
+        if (arguments().metaData().contains(QStringLiteral("compressionLevel"))) {
+            opts[QStringLiteral("CompressionLevel")] = arguments().metaData()[QStringLiteral("compressionLevel")];
+        }
+        if (arguments().metaData().contains(QStringLiteral("volumeSize"))) {
+            opts[QStringLiteral("VolumeSize")] = arguments().metaData()[QStringLiteral("volumeSize")];
+        }
+        m_model->archive()->setCompressionOptions(opts);
+    } else {
+        opts = m_model->archive()->compressionOptions();
+    }
+
+    qCDebug(ARK) << "Opening AddDialog with opts:" << opts;
+
     // #264819: passing widget() as the parent will not work as expected.
     //          KFileDialog will create a KFileWidget, which runs an internal
     //          event loop to stat the given directory. This, in turn, leads to
@@ -1128,24 +1268,39 @@ void Part::slotAddFiles()
     //          When KFileDialog::exec() is called, the widget is already shown
     //          and nothing happens.
 
-    const QStringList filesToAdd = QFileDialog::getOpenFileNames(widget(), i18nc("@title:window", "Add Files"));
+    QPointer<AddDialog> dlg = new AddDialog(widget(),
+                                            i18nc("@title:window", "Add Files"),
+                                            m_lastUsedAddPath,
+                                            m_model->archive()->mimeType(),
+                                            opts);
 
-    slotAddFiles(filesToAdd);
-}
-
-void Part::slotAddDir()
-{
-    const QString dirToAdd = QFileDialog::getExistingDirectory(widget(), i18nc("@title:window", "Add Folder"));
-
-    if (!dirToAdd.isEmpty()) {
-        slotAddFiles(QStringList() << dirToAdd);
+    if (dlg->exec() == QDialog::Accepted) {
+        qCDebug(ARK) << "Selected files:" << dlg->selectedFiles();
+        qCDebug(ARK) << "Options:" << dlg->compressionOptions();
+        m_model->archive()->setCompressionOptions(dlg->compressionOptions());
+        slotAddFiles(dlg->selectedFiles(), QString());
     }
+    delete dlg;
 }
 
 void Part::slotAddFilesDone(KJob* job)
 {
     if (job->error() && job->error() != KJob::KilledJobError) {
         KMessageBox::error(widget(), job->errorString());
+    } else {
+        // Hide the "archive will be created as soon as you add a file" message.
+        m_messageWidget->hide();
+
+        // For multi-volume archive, we need to re-open the archive after adding files
+        // because the name changes from e.g name.rar to name.part1.rar.
+        if (m_model->archive()->isMultiVolume()) {
+            qCDebug(ARK) << "Multi-volume archive detected, re-opening...";
+            KParts::OpenUrlArguments args = arguments();
+            args.metaData()[QStringLiteral("createNewArchive")] = QStringLiteral("false");
+            setArguments(args);
+
+            openUrl(QUrl::fromLocalFile(m_model->archive()->multiVolumeName()));
+        }
     }
 }
 
@@ -1184,8 +1339,12 @@ void Part::slotDeleteFiles()
 
 void Part::slotShowProperties()
 {
+    m_model->countEntriesAndSize();
     QPointer<Kerfuffle::PropertiesDialog> dialog(new Kerfuffle::PropertiesDialog(0,
-                                                                                 m_model->archive()));
+                                                                                 m_model->archive(),
+                                                                                 m_model->numberOfFiles(),
+                                                                                 m_model->numberOfFolders(),
+                                                                                 m_model->uncompressedSize()));
     dialog.data()->show();
 }
 
@@ -1256,11 +1415,11 @@ void Part::slotShowContextMenu()
 
 void Part::displayMsgWidget(KMessageWidget::MessageType type, const QString& msg)
 {
-    KMessageWidget *msgWidget = new KMessageWidget();
-    msgWidget->setText(msg);
-    msgWidget->setMessageType(type);
-    m_vlayout->insertWidget(0, msgWidget);
-    msgWidget->animatedShow();
+    // The widget could be already visible, so hide it.
+    m_messageWidget->hide();
+    m_messageWidget->setText(msg);
+    m_messageWidget->setMessageType(type);
+    m_messageWidget->animatedShow();
 }
 
 } // namespace Ark
