@@ -96,6 +96,7 @@ private slots:
     void slotLoadingStarted();
     void slotLoadingFinished(KJob *job);
     void slotOpenExtractedEntry(KJob*);
+    void slotPreviewExtractedEntry(KJob* job);
     void slotOpenEntry(int mode);
     void slotError(const QString& errorMessage, const QString& details);
     void slotExtractArchive();
@@ -104,8 +105,8 @@ private slots:
     void slotQuickExtractFiles(QAction*);
     void slotAddFiles();
     void slotAddFiles(const QStringList& files, const QString& path = QString());
-    void slotAddDir();
     void slotAddFilesDone(KJob*);
+    void slotTestingDone(KJob*);
     void slotDeleteFiles();
     void slotDeleteFilesDone(KJob*);
     void slotShowProperties();
@@ -121,6 +122,10 @@ private slots:
     void setReadyGui();
     void setFileNameFromArchive();
     void slotWatchedFileModified(const QString& file);
+    void slotShowComment();
+    void slotAddComment();
+    void slotCommentChanged();
+    void slotTestArchive();
 
 signals:
     void busy();
@@ -128,6 +133,7 @@ signals:
     void quit();
 
 private:
+    void resetGui();
     void setupView();
     void setupActions();
     bool isSingleFolderArchive() const;
@@ -146,16 +152,18 @@ private:
     QAction *m_extractArchiveAction;
     QAction *m_extractAction;
     QAction *m_addFilesAction;
-    QAction *m_addDirAction;
     QAction *m_deleteFilesAction;
     QAction *m_saveAsAction;
     QAction *m_propertiesAction;
+    QAction *m_editCommentAction;
+    QAction *m_testArchiveAction;
     KToggleAction *m_showInfoPanelAction;
     InfoPanel            *m_infoPanel;
     QSplitter            *m_splitter;
     QList<QTemporaryDir*>      m_tmpOpenDirList;
     bool                  m_busy;
     OpenFileMode m_openFileMode;
+    QUrl m_lastUsedAddPath;
 
     KAbstractWidgetJobTracker  *m_jobTracker;
     KParts::StatusBarExtension *m_statusBarExtension;
@@ -165,6 +173,8 @@ private:
     QSplitter *m_commentSplitter;
     QGroupBox *m_commentBox;
     QPlainTextEdit *m_commentView;
+    KMessageWidget *m_commentMsgWidget;
+    KMessageWidget *m_messageWidget;
 };
 
 } // namespace Ark
