@@ -60,7 +60,7 @@ ParameterList CliPlugin::parameterList() const
 
     if (p.isEmpty()) {
         //p[CaptureProgress] = true;
-        p[ListProgram] = p[ExtractProgram] = p[DeleteProgram] = p[AddProgram] = QStringList() << QStringLiteral("7z");
+        p[ListProgram] = p[ExtractProgram] = p[DeleteProgram] = p[AddProgram] = p[TestProgram] = QStringList() << QStringLiteral("7z");
         p[ListArgs] = QStringList() << QStringLiteral("l")
                                     << QStringLiteral("-slt")
                                     << QStringLiteral("$PasswordSwitch")
@@ -72,17 +72,23 @@ ParameterList CliPlugin::parameterList() const
         p[PreservePathSwitch] = QStringList() << QStringLiteral("x")
                                               << QStringLiteral("e");
         p[PasswordSwitch] = QStringList() << QStringLiteral("-p$Password");
-        p[EncryptHeaderSwitch] = QStringList() << QStringLiteral("$Enabled");
+        p[PasswordHeaderSwitch] = QStringList { QStringLiteral("-p$Password"), QStringLiteral("-mhe=on") };
         p[WrongPasswordPatterns] = QStringList() << QStringLiteral("Wrong password");
+        p[CompressionLevelSwitch] = QStringLiteral("-mx=$CompressionLevel");
         p[AddArgs] = QStringList() << QStringLiteral("a")
                                    << QStringLiteral("$Archive")
-                                   << QStringLiteral("$EncryptHeaderSwitch")
                                    << QStringLiteral("$PasswordSwitch")
+                                   << QStringLiteral("$CompressionLevelSwitch")
+                                   << QStringLiteral("$MultiVolumeSwitch")
                                    << QStringLiteral("$Files");
         p[DeleteArgs] = QStringList() << QStringLiteral("d")
                                       << QStringLiteral("$PasswordSwitch")
                                       << QStringLiteral("$Archive")
                                       << QStringLiteral("$Files");
+        p[TestArgs] = QStringList() << QStringLiteral("t")
+                                    << QStringLiteral("$Archive")
+                                    << QStringLiteral("$PasswordSwitch");
+        p[TestPassedPattern] = QStringLiteral("^Everything is Ok$");
 
         p[FileExistsExpression] = QStringList()
             << QStringLiteral("^\\(Y\\)es / \\(N\\)o / \\(A\\)lways / \\(S\\)kip all / A\\(u\\)to rename all / \\(Q\\)uit\\? $")
@@ -99,6 +105,8 @@ ParameterList CliPlugin::parameterList() const
         p[CorruptArchivePatterns] = QStringList() << QStringLiteral("Unexpected end of archive")
                                                   << QStringLiteral("Headers Error");
         p[DiskFullPatterns] = QStringList() << QStringLiteral("No space left on device");
+        p[MultiVolumeSwitch] = QStringLiteral("-v$VolumeSizek");
+        p[MultiVolumeSuffix] = QStringList() << QStringLiteral("$Suffix.001");
     }
 
     return p;
@@ -155,12 +163,15 @@ bool CliPlugin::readListLine(const QString& line)
                 m_archiveType = ArchiveTypeZip;
             } else if (type == QLatin1String("Rar")) {
                 m_archiveType = ArchiveTypeRar;
+            } else if (type == QLatin1String("Split")) {
+                setMultiVolume(true);
             } else {
                 // Should not happen
                 qCWarning(ARK) << "Unsupported archive type";
                 return false;
             }
-
+        } else if (line.startsWith(QStringLiteral("Volumes = "))) {
+            m_numberOfVolumes = line.section(QLatin1Char('='), 1).trimmed().toInt();
         } else if (rxComment.match(line).hasMatch()) {
             m_parseState = ParseStateComment;
             m_comment.append(line.section(QLatin1Char('='), 1) + QLatin1Char('\n'));
@@ -235,4 +246,20 @@ bool CliPlugin::readListLine(const QString& line)
     return true;
 }
 
+QStringList CliPlugin::passwordHeaderSwitch(const QString& password) const
+{
+    if (password.isEmpty()) {
+        return QStringList();
+    }
+
+    Q_ASSERT(m_param.contains(PasswordHeaderSwitch));
+
+    QStringList passwordHeaderSwitch = m_param.value(PasswordHeaderSwitch).toStringList();
+    Q_ASSERT(!passwordHeaderSwitch.isEmpty() && passwordHeaderSwitch.size() == 2);
+
+    passwordHeaderSwitch[0].replace(QLatin1String("$Password"), password);
+
+    return passwordHeaderSwitch;
+}
+
 #include "cliplugin.moc"
