@@ -38,7 +38,6 @@ CliPlugin::CliPlugin(QObject *parent, const QVariantList& args)
         , m_parseState(ParseStateTitle)
         , m_isUnrar5(false)
         , m_isPasswordProtected(false)
-        , m_isMultiVolume(false)
         , m_isSolid(false)
         , m_remainingIgnoreLines(1) //The first line of UNRAR output is empty.
         , m_linesComment(0)
@@ -78,7 +77,7 @@ ParameterList CliPlugin::parameterList() const
 
     if (p.isEmpty()) {
         p[CaptureProgress] = true;
-        p[ListProgram] = p[ExtractProgram] = QStringList() << QStringLiteral( "unrar" );
+        p[ListProgram] = p[ExtractProgram] = p[TestProgram] = QStringList() << QStringLiteral( "unrar" );
         p[DeleteProgram] = p[AddProgram] = QStringList() << QStringLiteral( "rar" );
 
         p[ListArgs] = QStringList() << QStringLiteral("vt")
@@ -89,14 +88,14 @@ ParameterList CliPlugin::parameterList() const
                                        << QStringLiteral( "-p-" )
                                        << QStringLiteral( "$PreservePathSwitch" )
                                        << QStringLiteral( "$PasswordSwitch" )
-                                       << QStringLiteral( "$RootNodeSwitch" )
                                        << QStringLiteral( "$Archive" )
                                        << QStringLiteral( "$Files" );
         p[PreservePathSwitch] = QStringList() << QStringLiteral( "x" )
                                               << QStringLiteral( "e" );
-        p[RootNodeSwitch] = QStringList() << QStringLiteral( "-ap$Path" );
         p[PasswordSwitch] = QStringList() << QStringLiteral( "-p$Password" );
         p[PasswordHeaderSwitch] = QStringList() << QStringLiteral("-hp$Password");
+        p[CompressionLevelSwitch] = QStringLiteral("-m$CompressionLevel");
+        p[MultiVolumeSwitch] = QStringLiteral("-v$VolumeSizek");
         p[DeleteArgs] = QStringList() << QStringLiteral( "d" )
                                       << QStringLiteral( "$PasswordSwitch" )
                                       << QStringLiteral( "$Archive" )
@@ -113,6 +112,8 @@ ParameterList CliPlugin::parameterList() const
         p[AddArgs] = QStringList() << QStringLiteral( "a" )
                                    << QStringLiteral( "$Archive" )
                                    << QStringLiteral("$PasswordSwitch")
+                                   << QStringLiteral("$CompressionLevelSwitch")
+                                   << QStringLiteral("$MultiVolumeSwitch")
                                    << QStringLiteral( "$Files" );
         p[PasswordPromptPattern] = QLatin1String("Enter password \\(will not be echoed\\) for");
         p[WrongPasswordPatterns] = QStringList() << QStringLiteral("password incorrect") << QStringLiteral("wrong password");
@@ -121,6 +122,18 @@ ParameterList CliPlugin::parameterList() const
         p[CorruptArchivePatterns] = QStringList() << QStringLiteral("Unexpected end of archive")
                                                   << QStringLiteral("the file header is corrupt");
         p[DiskFullPatterns] = QStringList() << QStringLiteral("No space left on device");
+        p[CommentArgs] = QStringList() << QStringLiteral("c")
+                                       << QStringLiteral("$CommentSwitch")
+                                       << QStringLiteral("$Archive");
+        p[CommentSwitch] = QStringLiteral("-z$CommentFile");
+        p[TestArgs] = QStringList() << QStringLiteral("t")
+                                    << QStringLiteral("$Archive")
+                                    << QStringLiteral("$PasswordSwitch");
+        p[TestPassedPattern] = QStringLiteral("^All OK$");
+        // rar will sometimes create multi-volume archives where first volume is
+        // called name.part1.rar and other times name.part01.rar.
+        p[MultiVolumeSuffix] = QStringList() << QStringLiteral("part01.$Suffix")
+                                             << QStringLiteral("part1.$Suffix");
     }
 
     return p;
@@ -199,9 +212,12 @@ void CliPlugin::handleUnrar5Line(const QString &line) {
         // "Details: " indicates end of header.
         if (line.startsWith(QStringLiteral("Details: "))) {
             ignoreLines(1, ParseStateEntryDetails);
-            if (line.contains(QLatin1String("volume")) && !m_isMultiVolume) {
-                m_isMultiVolume = true;
-                qCDebug(ARK) << "Multi-volume archive detected";
+            if (line.contains(QLatin1String("volume"))) {
+                m_numberOfVolumes++;
+                if (!isMultiVolume()) {
+                    setMultiVolume(true);
+                    qCDebug(ARK) << "Multi-volume archive detected";
+                }
             }
             if (line.contains(QLatin1String("solid")) && !m_isSolid) {
                 m_isSolid = true;
@@ -301,9 +317,12 @@ void CliPlugin::handleUnrar4Line(const QString &line) {
 
         if (rxCommentEnd.match(line).hasMatch()) {
 
-            if (line.startsWith(QLatin1String("Volume")) && !m_isMultiVolume) {
-                m_isMultiVolume = true;
-                qCDebug(ARK) << "Multi-volume archive detected";
+            if (line.startsWith(QLatin1String("Volume "))) {
+                m_numberOfVolumes++;
+                if (!isMultiVolume()) {
+                    setMultiVolume(true);
+                    qCDebug(ARK) << "Multi-volume archive detected";
+                }
             }
             if (line.startsWith(QLatin1String("Solid archive")) && !m_isSolid) {
                 m_isSolid = true;
@@ -331,6 +350,8 @@ void CliPlugin::handleUnrar4Line(const QString &line) {
         // Horizontal line indicates end of header.
         if (line.startsWith(QStringLiteral("--------------------"))) {
             m_parseState = ParseStateEntryFileName;
+        } else if (line.startsWith(QLatin1String("Volume "))) {
+            m_numberOfVolumes++;
         }
         return;
     }
