@@ -50,11 +50,9 @@ bool CliPlugin::list()
     cacheParameterList();
     m_operationMode = List;
 
-    QStringList args = m_param.value(ListArgs).toStringList();
-    substituteListVariables(args);
+    const auto args = substituteListVariables(m_param.value(ListArgs).toStringList(), password());
 
     if (!runProcess(m_param.value(ListProgram).toStringList(), args)) {
-        failOperation();
         return false;
     }
 
@@ -65,7 +63,7 @@ bool CliPlugin::list()
             qCWarning(ARK) << "Wrong password, list() aborted";
             emit error(i18n("Wrong password."));
             emit finished(false);
-            failOperation();
+            killProcess();
             setPassword(QString());
             return false;
         }
@@ -77,7 +75,6 @@ bool CliPlugin::list()
         }
     }
 
-    emit finished(true);
     return true;
 }
 
