@@ -40,7 +40,6 @@ K_PLUGIN_FACTORY_WITH_JSON(ReadWriteLibarchivePluginFactory, "kerfuffle_libarchi
 
 ReadWriteLibarchivePlugin::ReadWriteLibarchivePlugin(QObject *parent, const QVariantList & args)
     : LibarchivePlugin(parent, args)
-    , m_workDir(QDir::current())
 {
     qCDebug(ARK) << "Loaded libarchive read-write plugin";
 }
@@ -51,16 +50,9 @@ ReadWriteLibarchivePlugin::~ReadWriteLibarchivePlugin()
 
 bool ReadWriteLibarchivePlugin::addFiles(const QStringList& files, const CompressionOptions& options)
 {
-    qCDebug(ARK) << "Adding files" << files << "with CompressionOptions" << options;
+    qCDebug(ARK) << "Adding" << files.size() << "entries with CompressionOptions" << options;
 
     const bool creatingNewFile = !QFileInfo::exists(filename());
-    const QString globalWorkDir = options.value(QStringLiteral( "GlobalWorkDir" )).toString();
-
-    if (!globalWorkDir.isEmpty()) {
-        qCDebug(ARK) << "GlobalWorkDir is set, changing dir to " << globalWorkDir;
-        m_workDir.setPath(globalWorkDir);
-        QDir::setCurrent(globalWorkDir);
-    }
 
     m_writtenFiles.clear();
 
@@ -106,7 +98,7 @@ bool ReadWriteLibarchivePlugin::addFiles(const QStringList& files, const Compres
     archive_write_set_format_pax_restricted(arch_writer.data());
 
     int ret;
-    bool isLrzip = false;
+    bool requiresExecutable = false;
     if (creatingNewFile) {
         if (filename().right(2).toUpper() == QLatin1String("GZ")) {
             qCDebug(ARK) << "Detected gzip compression for new file";
@@ -132,7 +124,13 @@ bool ReadWriteLibarchivePlugin::addFiles(const QStringList& files, const Compres
         } else if (filename().right(3).toUpper() == QLatin1String("LRZ")) {
             qCDebug(ARK) << "Detected lrzip compression for new file";
             ret = archive_write_add_filter_lrzip(arch_writer.data());
-            isLrzip = true;
+            requiresExecutable = true;
+#ifdef HAVE_LIBARCHIVE_3_2_0
+        } else if (filename().right(3).toUpper() == QLatin1String("LZ4")) {
+            qCDebug(ARK) << "Detected lz4 compression for new file";
+            ret = archive_write_add_filter_lz4(arch_writer.data());
+            requiresExecutable = true;
+#endif
         } else if (filename().right(3).toUpper() == QLatin1String("TAR")) {
             qCDebug(ARK) << "Detected no compression for new file (pure tar)";
             ret = archive_write_add_filter_none(arch_writer.data());
@@ -142,12 +140,13 @@ bool ReadWriteLibarchivePlugin::addFiles(const QStringList& files, const Compres
         }
 
         // Libarchive emits a warning for lrzip due to using external executable.
-        if ((isLrzip && ret != ARCHIVE_WARN) ||
-            (!isLrzip && ret != ARCHIVE_OK)) {
+        if ((requiresExecutable && ret != ARCHIVE_WARN) ||
+            (!requiresExecutable && ret != ARCHIVE_OK)) {
             emit error(xi18nc("@info", "Setting the compression method failed with the following error:<nl/><message>%1</message>",
                               QLatin1String(archive_error_string(arch_writer.data()))));
             return false;
         }
+
     } else {
         switch (archive_filter_code(arch_reader.data(), 0)) {
         case ARCHIVE_FILTER_GZIP:
@@ -173,8 +172,14 @@ bool ReadWriteLibarchivePlugin::addFiles(const QStringList& files, const Compres
             break;
         case ARCHIVE_FILTER_LRZIP:
             ret = archive_write_add_filter_lrzip(arch_writer.data());
-            isLrzip = true;
+            requiresExecutable = true;
             break;
+#ifdef HAVE_LIBARCHIVE_3_2_0
+        case ARCHIVE_FILTER_LZ4:
+            ret = archive_write_add_filter_lz4(arch_writer.data());
+            requiresExecutable = true;
+            break;
+#endif
         case ARCHIVE_FILTER_NONE:
             ret = archive_write_add_filter_none(arch_writer.data());
             break;
@@ -185,14 +190,26 @@ bool ReadWriteLibarchivePlugin::addFiles(const QStringList& files, const Compres
         }
 
         // Libarchive emits a warning for lrzip due to using external executable.
-        if ((isLrzip && ret != ARCHIVE_WARN) ||
-            (!isLrzip && ret != ARCHIVE_OK)) {
+        if ((requiresExecutable && ret != ARCHIVE_WARN) ||
+            (!requiresExecutable && ret != ARCHIVE_OK)) {
             emit error(xi18nc("@info", "Setting the compression method failed with the following error:<nl/><message>%1</message>",
                               QLatin1String(archive_error_string(arch_writer.data()))));
             return false;
         }
     }
 
+    // Set compression level if passed in CompressionOptions.
+    if (options.contains(QStringLiteral("CompressionLevel"))) {
+        qCDebug(ARK) << "Using compression level:" << options.value(QStringLiteral("CompressionLevel")).toString();
+        ret = archive_write_set_filter_option(arch_writer.data(), NULL, "compression-level", options.value(QStringLiteral("CompressionLevel")).toString().toUtf8());
+        if (ret != ARCHIVE_OK) {
+            qCWarning(ARK) << "Failed to set compression level";
+            emit error(xi18nc("@info", "Setting the compression level failed with the following error:<nl/><message>%1</message>",
+                              QLatin1String(archive_error_string(arch_writer.data()))));
+            return false;
+        }
+    }
+
     ret = archive_write_open_fd(arch_writer.data(), tempFile.handle());
     if (ret != ARCHIVE_OK) {
         emit error(xi18nc("@info", "Opening the archive for writing failed with the following error:<nl/><message>%1</message>",
@@ -201,6 +218,8 @@ bool ReadWriteLibarchivePlugin::addFiles(const QStringList& files, const Compres
     }
 
     // First write the new files.
+    qCDebug(ARK) << "Writing new entries";
+    int no_entries = 0;
     foreach(const QString& selectedFile, files) {
 
         if (m_abortOperation) {
@@ -210,6 +229,7 @@ bool ReadWriteLibarchivePlugin::addFiles(const QStringList& files, const Compres
         if (!writeFile(selectedFile, arch_writer.data())) {
             return false;
         }
+        no_entries++;
 
         // For directories, write all subfiles/folders.
         if (QFileInfo(selectedFile).isDir()) {
@@ -235,15 +255,20 @@ bool ReadWriteLibarchivePlugin::addFiles(const QStringList& files, const Compres
                 if (!writeFile(path, arch_writer.data())) {
                     return false;
                 }
+                no_entries++;
             }
         }
     }
+    qCDebug(ARK) << "Added" << no_entries << "new entries to archive";
 
     struct archive_entry *entry;
 
     // If we have old archive entries.
     if (!creatingNewFile) {
 
+        qCDebug(ARK) << "Copying any old entries";
+        no_entries = 0;
+
         // Copy old entries from previous archive to new archive.
         while (!m_abortOperation && (archive_read_next_header(arch_reader.data(), &entry) == ARCHIVE_OK)) {
 
@@ -262,6 +287,7 @@ bool ReadWriteLibarchivePlugin::addFiles(const QStringList& files, const Compres
                 // If the whole archive is extracted and the total filesize is
                 // available, we use partial progress.
                 copyData(QLatin1String(archive_entry_pathname(entry)), arch_reader.data(), arch_writer.data(), false);
+                no_entries++;
                 break;
             case ARCHIVE_FAILED:
             case ARCHIVE_FATAL:
@@ -278,6 +304,7 @@ bool ReadWriteLibarchivePlugin::addFiles(const QStringList& files, const Compres
             }
             archive_entry_clear(entry);
         }
+        qCDebug(ARK) << "Added" << no_entries << "old entries to archive";
     }
 
     m_abortOperation = false;
@@ -295,7 +322,7 @@ bool ReadWriteLibarchivePlugin::addFiles(const QStringList& files, const Compres
 
 bool ReadWriteLibarchivePlugin::deleteFiles(const QVariantList& files)
 {
-   qCDebug(ARK) << "Deleting files" << files;
+   qCDebug(ARK) << "Deleting" << files.size() << "entries";
 
     ArchiveRead arch_reader(archive_read_new());
     if (!(arch_reader.data())) {
@@ -372,15 +399,18 @@ bool ReadWriteLibarchivePlugin::deleteFiles(const QVariantList& files)
 
     struct archive_entry *entry;
 
+
+
     // Copy old elements from previous archive to new archive.
-    while (!m_abortOperation && (archive_read_next_header(arch_reader.data(), &entry) == ARCHIVE_OK)) {
+    int no_entries = 0;
+    while (archive_read_next_header(arch_reader.data(), &entry) == ARCHIVE_OK) {
 
         const QString entryName = QFile::decodeName(archive_entry_pathname(entry));
 
         if (files.contains(entryName)) {
             archive_read_data_skip(arch_reader.data());
-            qCDebug(ARK) << "Entry to be deleted, skipping" << entryName;
             emit entryRemoved(entryName);
+            no_entries++;
             continue;
         }
 
@@ -405,6 +435,7 @@ bool ReadWriteLibarchivePlugin::deleteFiles(const QVariantList& files)
             break;
         }
     }
+    qCDebug(ARK) << "Removed" << no_entries << "entries from archive";
 
     m_abortOperation = false;
 
@@ -421,17 +452,10 @@ bool ReadWriteLibarchivePlugin::deleteFiles(const QVariantList& files)
 
 // TODO: if we merge this with copyData(), we can pass more data
 //       such as an fd to archive_read_disk_entry_from_file()
-bool ReadWriteLibarchivePlugin::writeFile(const QString& fileName, struct archive* arch_writer)
+bool ReadWriteLibarchivePlugin::writeFile(const QString& relativeName, struct archive* arch_writer)
 {
     int header_response;
-
-    const bool trailingSlash = fileName.endsWith(QLatin1Char( '/' ));
-
-    // #191821: workDir must be used instead of QDir::current()
-    //          so that symlinks aren't resolved automatically
-    // TODO: this kind of call should be moved upwards in the
-    //       class hierarchy to avoid code duplication
-    const QString relativeName = m_workDir.relativeFilePath(fileName) + (trailingSlash ? QStringLiteral( "/" ) : QStringLiteral( "" ));
+    const QString absoluteFilename = QFileInfo(relativeName).absoluteFilePath();
 
     // #253059: Even if we use archive_read_disk_entry_from_file,
     //          libarchive may have been compiled without HAVE_LSTAT,
@@ -439,25 +463,24 @@ bool ReadWriteLibarchivePlugin::writeFile(const QString& fileName, struct archiv
     //          which case stat() will be called. To avoid this, we
     //          call lstat() ourselves.
     struct stat st;
-    lstat(QFile::encodeName(fileName).constData(), &st);
+    lstat(QFile::encodeName(absoluteFilename).constData(), &st);
 
     struct archive_entry *entry = archive_entry_new();
     archive_entry_set_pathname(entry, QFile::encodeName(relativeName).constData());
-    archive_entry_copy_sourcepath(entry, QFile::encodeName(fileName).constData());
+    archive_entry_copy_sourcepath(entry, QFile::encodeName(absoluteFilename).constData());
     archive_read_disk_entry_from_file(m_archiveReadDisk.data(), entry, -1, &st);
 
-    qCDebug(ARK) << "Writing new entry " << archive_entry_pathname(entry);
     if ((header_response = archive_write_header(arch_writer, entry)) == ARCHIVE_OK) {
         // If the whole archive is extracted and the total filesize is
         // available, we use partial progress.
-        copyData(fileName, arch_writer, false);
+        copyData(absoluteFilename, arch_writer, false);
     } else {
         qCCritical(ARK) << "Writing header failed with error code " << header_response;
         qCCritical(ARK) << "Error while writing..." << archive_error_string(arch_writer) << "(error no =" << archive_errno(arch_writer) << ')';
 
         emit error(xi18nc("@info Error in a message box",
                           "Ark could not compress <filename>%1</filename>:<nl/>%2",
-                          fileName,
+                          absoluteFilename,
                           QString::fromUtf8(archive_error_string(arch_writer))));
 
         archive_entry_free(entry);
